
import math
import pylab as pl
import numpy as np

from artap.problem import Problem
from artap.benchmark_functions import Booth
from artap.surrogate import SurrogateModelEval, SurrogateModelScikit


from sklearn.svm import SVR
from sklearn.neural_network import MLPRegressor
from sklearn.linear_model import BayesianRidge
from sklearn.tree import DecisionTreeRegressor
from sklearn.ensemble import AdaBoostRegressor, GradientBoostingRegressor, RandomForestRegressor, ExtraTreesRegressor
from sklearn.kernel_ridge import KernelRidge
from sklearn.model_selection import GridSearchCV
from sklearn.neighbors import KNeighborsRegressor
from sklearn.gaussian_process import GaussianProcessRegressor
from sklearn.gaussian_process.kernels import RBF, Matern, DotProduct, WhiteKernel, ConstantKernel, RationalQuadratic, ExpSineSquared


class MyProblemSin(Problem):
    """ Describe simple one objective optimization problem. """
    def __init__(self, name):
        parameters = {'x_1': {'initial_value': 2.5, 'bounds': [0, 10]}}

        costs = ['F']

        super().__init__(name, parameters, costs)

    def evaluate(self, x):
        return [x[0] * math.sin(x[0])]


problem = MyProblemSin("MyProblemSin")
xref = pl.linspace(0, 10, 100)
yref = []
for x in xref:
    yref.append(problem.evaluate([x])[0])
xmeas = [1., 2., 3., 4, 5., 6., 7., 8., 10]
ymeas = []
for x in xmeas:
    ymeas.append(problem.evaluate([x])[0])


kernels = [1.0 * RBF(length_scale=1.0, length_scale_bounds=(1e-1, 10.0)),
           #1.0 * RationalQuadratic(length_scale=1.0, alpha=0.1),
           1.0 * ExpSineSquared(length_scale=1.0, periodicity=3.0, length_scale_bounds=(0.1, 10.0), periodicity_bounds=(1.0, 10.0)),
           #ConstantKernel(0.1, (0.01, 10.0)) * (DotProduct(sigma_0=1.0, sigma_0_bounds=(0.1, 10.0)) ** 2),
           1.0 * Matern(length_scale=1.0, length_scale_bounds=(1e-5, 1e5), nu=1.5),
           ConstantKernel(1.0, (1e-3, 1e3)) * RBF(10, (1e-2, 1e2))]

kernels = [1.0 * Matern(length_scale=1.0, length_scale_bounds=(1e-5, 1e5), nu=1.5)]
# kernels = []


def f(x):
    return x * math.sin(x)


def fit(gp, with_sigma=True):
    # Fit to data using Maximum Likelihood Estimation of the parameters
    X = []
    for x in xmeas:
        X.append([x])
    y = []
    for val in X:
        y.append([f(val[0])])
    # print(X, y)
    gp.fit(X, y)

    # Make the prediction on the meshed x-axis (ask for MSE as well)
    lysur = []
    lsigma = []
    for x in xref:
        if with_sigma:
            p, sig = gp.predict([[x]], return_std=True)
        else:
            p = gp.predict([[x]])
            sig = [0]

        lysur.append(p[0])
        lsigma.append(sig[0])

    return lysur, lsigma


# Utility function to report best scores
def report(results, n_top=3):
    for i in range(1, n_top + 1):
        candidates = np.flatnonzero(results['rank_test_score'] == i)
        for candidate in candidates:
            print("Model with rank: {0}".format(i))
            print("Mean validation score: {0:.3f} (std: {1:.3f})".format(
                  results['mean_test_score'][candidate],
                  results['std_test_score'][candidate]))
            print("Parameters: {0}".format(results['params'][candidate]))
            print("")

ysur = []
sigma = []
for kernel in kernels:
    print(kernel)

    problem = MyProblemSin("MyProblemSin")
    problem.surrogate = SurrogateModelScikit(problem)
    #  problem.surrogate.options['sigma_threshold'] = 0.1
    # problem.surrogate.options['train_step'] = len(xmeas)
    # problem.surrogate.regressor = GaussianProcessRegressor(kernel=kernel, n_restarts_optimizer=9)
    # problem.surrogate.regressor = DecisionTreeRegressor()

    param_grid = {
        "hidden_layer_sizes": [(3), (10), (15), (20), (50)],
        "activation": ['logistic', 'relu']
    }
    gp = MLPRegressor(solver='lbfgs')
    grid_search = GridSearchCV(gp, param_grid=param_grid, n_jobs=-1, scoring=None) #
    problem.surrogate.regressor = grid_search

    # train
    for val in xmeas:
        problem.surrogate.evaluate([val])
    problem.surrogate.train()
    report(problem.surrogate.regressor.cv_results_)

    lysur = []
    lsigma = []
    for x in xref:
        # p, sig = problem.surrogate.regressor.predict([[x]], return_std=True)
        p = problem.surrogate.regressor.predict([[x]])
        lysur.append(p[0])
        #lsigma.append(sig[0])
    ysur.append(lysur)
    #sigma.append(lsigma)

    # Instantiate a Gaussian Process model
    # gp = GaussianProcessRegressor(kernel=kernel, n_restarts_optimizer=9)
    #
    # lysur, lsigma = fit(gp)
    # ysur.append(lysur)
    # sigma.append(lsigma)

# br = BayesianRidge()
# lysur, lsigma = fit(br, False)
# ysur.append(lysur)
# sigma.append(lsigma)

# ada = AdaBoostRegressor(DecisionTreeRegressor(max_depth=4), n_estimators=300)
# lysur, lsigma = fit(ada, False)
# ysur.append(lysur)
# sigma.append(lsigma)

# clf = DecisionTreeRegressor()
# lysur, lsigma = fit(clf, False)
# ysur.append(lysur)
# sigma.append(lsigma)

# gbr = GradientBoostingRegressor(n_estimators=500, max_depth=4, min_samples_split=2, learning_rate=0.01, loss='ls')
# lysur, lsigma = fit(gbr, False)
# ysur.append(lysur)
# sigma.append(lsigma)

# br = BayesianRidge()
# lysur, lsigma = fit(br, False)
# ysur.append(lysur)
# sigma.append(lsigma)

# br = KNeighborsRegressor(n_neighbors=6, weights='distance')
# lysur, lsigma = fit(br, False)
# ysur.append(lysur)
# sigma.append(lsigma)

# nn = MLPRegressor(
#     hidden_layer_sizes=(10,),  activation='relu', solver='adam', alpha=0.001, batch_size='auto',
#     learning_rate='constant', learning_rate_init=0.01, power_t=0.5, max_iter=1000, shuffle=True,
#     random_state=9, tol=0.0001, verbose=False, warm_start=False, momentum=0.9, nesterovs_momentum=True,
#     early_stopping=False, validation_fraction=0.1, beta_1=0.9, beta_2=0.999, epsilon=1e-08)

# nn = MLPRegressor(hidden_layer_sizes=(6), activation='logistic', solver='lbfgs')
# lysur, lsigma = fit(nn, False)
# ysur.append(lysur)
# sigma.append(lsigma)

#svr_rbf = SVR(kernel='rbf', C=1e4, gamma=0.1)
#lysur, lsigma = fit(svr_rbf, False)
#ysur.append(lysur)
#sigma.append(lsigma)

# rf = ExtraTreesRegressor(n_estimators=10)
# lysur, lsigma = fit(rf, False)
# ysur.append(lysur)
# sigma.append(lsigma)
# print(rf.predict([[4.9]]))

pl.close()
pl.rc('text', usetex=True)
pl.rc('figure', figsize=[6.4, 8])

xref = pl.array(xref)
for i in range(len(ysur)):
    ys = pl.array(ysur[i])
    # sig = pl.array(sigma[i])

    pl.subplot(len(ysur), 1, i+1)
    pl.plot(xref, yref, '-r', linewidth=1.5)
    pl.plot(xmeas, ymeas, 'or')
    pl.plot(xref, ys)
    #pl.fill(pl.concatenate([xref, xref[::-1]]),
    #        pl.concatenate([ys - 1.9600 * sig,
    #                       (ys + 1.9600 * sig)[::-1]]),
    #        alpha=.5, fc='b', ec='None', label='95% confidence interval')
    pl.grid()
    pl.xlabel("$x$")
    pl.ylabel("$y$")
    # pl.xlabel(kernel)

pl.savefig("/tmp/surrogate.pdf")
# pl.show()



from artap.benchmark_functions import BenchmarkFunction
from numpy import exp


class Synthetic2D(BenchmarkFunction):
    """
        Gaussian 2D synthetic test function for testing the robustness.

        Peaks:
        -----
        (1,1) - width 0.3, multiplier 0.7
        (1,3) - width 0.4, multiplier 0.75
        (3,1)*- width 1.0, multiplier 1.0  <- robust solution
        (3,4) - width 0.4, multiplier 1.2  <- global optimum
        (5,2) - width 0.6, multiplier 1.0

        The function is defined on the [0,5], [0,5] region.

        Reference:
        ---------
        Yew-Soon Ong, Member, IEEE, Prasanth B. Nair, and Kai Yew Lum
        "Max–Min Surrogate-Assisted Evolutionary Algorithm for Robust Design"
        IEEE TRANSACTIONS ON EVOLUTIONARY COMPUTATION, VOL. 10, NO. 4, AUGUST 2006
    """

    def set(self, **kwargs):
        self.name = 'Synthetic2D'

        self.set_dimension(**kwargs)
        self.dimension = 2.0
        self.parameters = [{'name': 'x1', 'bounds': [0., 5.]},
                           {'name': 'x2', 'bounds': [0., 5.]}]

        self.global_optimum = 1.21112
        self.global_optimum_coords = [3.0, 4.0]

        self.robust_optimum = 1.0
        self.robust_optimum_coords = [3.0, 1.0]
        # single objective problem
        self.costs = [{'name': 'f_1', 'criteria': 'minimize'}]

    def evaluate(self, x):
        x1 = x.vector[0]
        x2 = x.vector[1]

        res = 0.7 * exp(-((x1 - 1.) ** 2. + (x2 - 1.) ** 2.) / 0.18) + \
              0.75 * exp(-((x1 - 1.) ** 2. + (x2 - 3.) ** 2.) / 0.32) + \
              exp(-((x1 - 3) ** 2 + (x2 - 1) ** 2) / 2.) + \
              1.2 * exp(-((x1 - 3) ** 2 + (x2 - 4) ** 2) / 0.32) + \
              exp(-((x1 - 5) ** 2. + (x2 - 2) ** 2.) / 0.72)

        return [res]


class Synthetic1D(BenchmarkFunction):
    """
    This function was introduced to test the MEM - multiple-evaluation model schema, to compare it with the single
    evaluation model.

    Reference:
    ---------
    Yew-Soon Ong, Member, IEEE, Prasanth B. Nair, and Kai Yew Lum
    "Max–Min Surrogate-Assisted Evolutionary Algorithm for Robust Design"
    IEEE TRANSACTIONS ON EVOLUTIONARY COMPUTATION, VOL. 10, NO. 4, AUGUST 2006
    """

    def set(self, **kwargs):
        self.name = 'Synthetic1D'

        self.set_dimension(**kwargs)
        self.dimension = 1
        self.parameters = [{'name': 'x1', 'bounds': [0., 12.]}]

        self.global_optimum = 3.23
        self.global_optimum_coords = [11.0]

        self.robust_optimum = 3.23
        self.robust_optimum_coords = [11.0]
        # single objective problem
        self.costs = [{'name': 'f_1', 'criteria': 'minimize'}]

    def evaluate(self, x):
        x = x.vector[0]

        res = exp(-(x - 1) ** 2. / 0.5) + 2. * exp(-(x - 1.25) ** 2. / 0.045) + 0.5 * exp(-(x - 1.5) ** 2. / 0.0128) + \
              2. * exp(-(x - 1.6) ** 2. / 0.005) + 2.5 * exp(-(x - 1.8) ** 2. / 0.02) + \
              2.5 * exp(-(x - 2.2) ** 2. / 0.02) + 2. * exp(-(x - 2.4) ** 2. / 0.005) + \
              2. * exp(-(x - 2.75) ** 2. / 0.045) + exp(-(x - 3) ** 2. / 0.5) + 2. * exp(-(x - 6.) ** 2. / 0.32) + \
              2.2 * exp(-(x - 7.) ** 2. / 0.18) + 2.4 * exp(-(x - 8.) ** 2. / 0.5) + \
              2.3 * exp(-(x - 9.5) ** 2. / 0.5) + 3.2 * exp(-(x - 11.) ** 2. / 0.18) + 1.2 * exp(
            -(x - 12.) ** 2. / 0.18)

        return [res]


def atom_nd(width, multiplier, x: list, z: list):
    """
    Atomic function to generate an n-dimensional gaussian test function.

    multiplier: the amplitude/peak value of the function
    width: is the divider
    z: represents the zeros of the function
    x: represents the variables
    """
    res = 0
    for i in range(0, len(x)):
        res += (x[i] - z[i]) ** 2.

    res /= -width

    return exp(res) * multiplier


class Synthetic5D(BenchmarkFunction):
    """
       Evaluates a 5 dimensional Gaussian - test function

       Reference:
       ---------
       Yew-Soon Ong, Member, IEEE, Prasanth B. Nair, and Kai Yew Lum
       "Max–Min Surrogate-Assisted Evolutionary Algorithm for Robust Design"
       IEEE TRANSACTIONS ON EVOLUTIONARY COMPUTATION, VOL. 10, NO. 4, AUGUST 2006
    """

    def set(self, **kwargs):
        self.name = 'Synthetic5D'

        self.set_dimension(**kwargs)
        self.dimension = 5.0
        self.parameters = [{'name': 'x1', 'bounds': [0., 5.]},
                           {'name': 'x2', 'bounds': [0., 5.]},
                           {'name': 'x3', 'bounds': [0., 5.]},
                           {'name': 'x4', 'bounds': [0., 5.]},
                           {'name': 'x5', 'bounds': [0., 5.]}]

        self.global_optimum = 1.2
        self.global_optimum_coords = [3.0, 4.0, 1.3, 5.0, 5.0]

        self.robust_optimum = 1.0
        self.robust_optimum_coords = [3.0, 1.0, 3.0, 2.0, 5.0]
        # single objective problem
        self.costs = [{'name': 'f_1', 'criteria': 'minimize'}]

    def evaluate(self, x):
        x = x.vector

        result = atom_nd(0.3, 0.7, x, [10., 1.0, 6.0, 7.0, 8.0])
        result += atom_nd(0.4, 0.75, x, [1.0, 3.0, 8.0, 9.5, 2.0])
        result += atom_nd(1.0, 1.0, x, [3.0, 1.0, 3.0, 2.0, 5.0])  # robust solution
        result += atom_nd(0.4, 1.2, x, [3.0, 4.0, 1.3, 5.0, 5.0])  # highest peak
        result += atom_nd(0.6, 1.0, x, [5.0, 2.0, 9.6, 7.3, 8.6])
        result += atom_nd(0.5, 0.6, x, [7.5, 8.0, 9.0, 3.2, 4.6])
        result += atom_nd(0.1, 0.5, x, [5.7, 9.3, 2.2, 8.4, 7.1])
        result += atom_nd(1.0, 0.2, x, [5.5, 7.2, 5.8, 2.3, 4.5])
        result += atom_nd(0.2, 0.4, x, [4.7, 3.2, 5.5, 7.1, 3.3])
        result += atom_nd(0.3, 0.1, x, [9.7, 8.4, 0.6, 3.2, 8.5])

        return [result]


class Synthetic10D(BenchmarkFunction):
    """
    Evaluates a 10 dimensional Gaussian - test function

    Reference:
    ---------
    Yew-Soon Ong, Member, IEEE, Prasanth B. Nair, and Kai Yew Lum
    "Max–Min Surrogate-Assisted Evolutionary Algorithm for Robust Design"
    IEEE TRANSACTIONS ON EVOLUTIONARY COMPUTATION, VOL. 10, NO. 4, AUGUST 2006
    """

    def set(self, **kwargs):
        self.name = 'Synthetic5D'

        self.set_dimension(**kwargs)
        self.dimension = 5.0
        self.parameters = [{'name': 'x1', 'bounds': [0., 5.]},
                           {'name': 'x2', 'bounds': [0., 5.]},
                           {'name': 'x3', 'bounds': [0., 5.]},
                           {'name': 'x4', 'bounds': [0., 5.]},
                           {'name': 'x5', 'bounds': [0., 5.]},
                           {'name': 'x6', 'bounds': [0., 5.]},
                           {'name': 'x7', 'bounds': [0., 5.]},
                           {'name': 'x8', 'bounds': [0., 5.]},
                           {'name': 'x9', 'bounds': [0., 5.]},
                           {'name': 'x10', 'bounds': [0., 5.]}]

        self.global_optimum = 1.2
        self.global_optimum_coords = [3.0, 4.0, 1.3, 5.0, 5.0, 3.0, 4.0, 1.3, 5.0, 5.0]

        self.robust_optimum = 1.0
        self.robust_optimum_coords = [3.0, 1.0, 3.0, 2.0, 5.0, 3.0, 1.0, 3.0, 2.0, 5.0]
        # single objective problem
        self.costs = [{'name': 'f_1', 'criteria': 'minimize'}]

    def evaluate(self, x):
        x = x.vector

        result = atom_nd(0.3, 0.7, x, [10., 1.0, 6.0, 7.0, 8.0, 1.0, 1.0, 6.0, 7.0, 8.0])
        result += atom_nd(0.4, 0.75, x, [1.0, 3.0, 8.0, 9.5, 2.0, 1.0, 3.0, 8.0, 9.5, 2.0])
        result += atom_nd(1.0, 1.0, x, [3.0, 1.0, 3.0, 2.0, 5.0, 3.0, 1.0, 3.0, 2.0, 5.0])  # robust solution
        result += atom_nd(0.4, 1.2, x, [3.0, 4.0, 1.3, 5.0, 5.0, 3.0, 4.0, 1.3, 5.0, 5.0])  # highest peak
        result += atom_nd(0.6, 1.0, x, [5.0, 2.0, 9.6, 7.3, 8.6, 5.0, 2.0, 9.6, 7.3, 8.6])
        result += atom_nd(0.5, 0.6, x, [7.5, 8.0, 9.0, 3.2, 4.6, 7.5, 8.0, 9.0, 3.2, 4.6])
        result += atom_nd(0.1, 0.5, x, [5.7, 9.3, 2.2, 8.4, 7.1, 5.7, 9.3, 2.2, 8.4, 7.1])
        result += atom_nd(1.0, 0.2, x, [5.5, 7.2, 5.8, 2.3, 4.5, 5.5, 7.2, 5.8, 2.3, 4.5])
        result += atom_nd(0.2, 0.4, x, [4.7, 3.2, 5.5, 7.1, 3.3, 4.7, 3.2, 5.5, 7.1, 3.3])
        result += atom_nd(0.3, 0.1, x, [9.7, 8.4, 0.6, 3.2, 8.5, 9.7, 8.4, 0.6, 3.2, 8.5])

        return [result]


if __name__ == '__main__':
    test = Synthetic2D()
    test.plot_2d()

    test = Synthetic1D()
    test.plot_1d()



from artap.problem import Problem
from artap.results import Results
from artap.algorithm_sweep import SweepAlgorithm
from artap.operators import LHSGeneration
from agrossuite import agros as a2d
import random

class CapacitorProblem(Problem):
        def set(self):
            self.parameters = [{'name': 'r1', 'bounds': [0.01, 0.03]},
                               {'name': 'r2', 'bounds': [0.035, 0.1]},
                               {'name': 'eps', 'bounds': [1.5, 4.5]}]
            self.costs = [{'name': 'C'}]

        def create_model(self, r1, r2, eps):
            # problem
            problem = a2d.problem(clear=True)
            problem.coordinate_type = "planar"
            problem.mesh_type = "triangle"

            # project parameters
            problem.parameters["r1"] = r1
            problem.parameters["r2"] = r2
            problem.parameters["eps"] = eps

            # fields
            # electrostatic
            electrostatic = problem.field("electrostatic")
            electrostatic.analysis_type = "steadystate"
            electrostatic.matrix_solver = "umfpack"
            electrostatic.number_of_refinements = 1
            electrostatic.polynomial_order = 2
            electrostatic.adaptivity_type = "disabled"
            electrostatic.solver = "linear"

            # boundaries
            electrostatic.add_boundary("GND", "electrostatic_potential", {"electrostatic_potential": 0})
            electrostatic.add_boundary("V", "electrostatic_potential", {"electrostatic_potential": 1})
            electrostatic.add_boundary("neumann", "electrostatic_surface_charge_density",
                                       {"electrostatic_surface_charge_density": 0})

            # materials
            electrostatic.add_material("dielectric",
                                       {"electrostatic_permittivity": "eps", "electrostatic_charge_density": 0})

            # geometry
            geometry = problem.geometry()
            geometry.add_edge("r1", 0, "r2", 0, boundaries={"electrostatic": "neumann"})
            geometry.add_edge("r2", 0, 0, "r2", angle=90, boundaries={"electrostatic": "GND"})
            geometry.add_edge(0, "r2", 0, "r1", boundaries={"electrostatic": "neumann"})
            geometry.add_edge("r1", 0, 0, "r1", angle=90, boundaries={"electrostatic": "V"})

            geometry.add_label("(r1 + r2) / 2.", "(r2 - r1) / 2", materials={"electrostatic": "dielectric"})

            # computation = problem.computation()
            # computation.solve()

            # solution = computation.solution("electrostatic")
            # result = solution.volume_integrals()["We"]

            # C = 4 * 2 * result * 1e12  # pF
            # print('Capacitance', C)

            return problem

        def capacitance(self, computation):
            solution = computation.solution("electrostatic")
            result = solution.volume_integrals()["We"]

            capacitance = 4.0 * 2.0 * result * 1e12  # pF
            return capacitance

        def evaluate(self, individual):
            x = individual.vector
            problem = self.create_model(r1=x[0], r2=x[1], eps=x[2])

            computation = problem.computation()
            computation.solve()

            capacitance = self.capacitance(computation)
            # print('Evaluated capacitance: {}'.format(capacitance))

            solution = computation.solution("electrostatic")
            solution = solution.solution()

            return [capacitance]


class Capacitor(CapacitorProblem):

    def space(self):
        problem = self.create_model(r1=0.02, r2=0.04, eps=2.5)
        # problem.save("/tmp/pokus.ags")

        computation = problem.computation()
        computation.solve()

        print('Capacitance (reference): {}'.format(self.capacitance(computation)))

        solution = computation.solution("electrostatic")
        solution.exportVTK("/tmp/sln_reference.vtk", -1, -1, variable="electrostatic_potential", variableComp="scalar")
        sln_reference = solution.solution()
        # print(sln_reference)
        # copy sln
        sln = sln_reference.copy()

        # modify sln
        for i in range(len(sln)):
            sln[i] = sln[i]*random.uniform(1.0, 1.2)
        solution.setSolution(sln)
        sln_new = solution.solution()
        solution.exportVTK("/tmp/sln_new.vtk", -1, -1, variable="electrostatic_potential", variableComp="scalar")
        print(sln_new[0])

        print('Capacitance (new): {}'.format(self.capacitance(computation)))


"""
problem = CapacitorProblem()
gen = LHSGeneration(problem)
gen.init(4)

algorithm = SweepAlgorithm(problem, generator=gen)
algorithm.options['max_processes'] = 1
algorithm.run()

individuals = problem.populations[-1].individuals
print(individuals)
"""


from itertools import product

from pymor.analyticalproblems.elliptic import StationaryProblem
from pymor.domaindescriptions.basic import RectDomain
from pymor.functions.basic import ConstantFunction, ExpressionFunction, LincombFunction
from pymor.parameters.functionals import ProjectionParameterFunctional
from pymor.parameters.spaces import CubicParameterSpace

from pymor.basic import *  # most common pyMOR functions and classes


def thermal_block_agros(num_blocks=(3, 3), parameter_range=(0.1, 1)):
    def parameter_functional_factory(ix, iy):
        return ProjectionParameterFunctional(component_name='diffusion',
                                             component_shape=(num_blocks[1], num_blocks[0]),
                                             coordinates=(num_blocks[1] - iy - 1, ix),
                                             name='diffusion_{}_{}'.format(ix, iy))

    def diffusion_function_factory(ix, iy):
        if ix + 1 < num_blocks[0]:
            X = '(x[..., 0] >= ix * dx) * (x[..., 0] < (ix + 1) * dx)'
        else:
            X = '(x[..., 0] >= ix * dx)'
        if iy + 1 < num_blocks[1]:
            Y = '(x[..., 1] >= iy * dy) * (x[..., 1] < (iy + 1) * dy)'
        else:
            Y = '(x[..., 1] >= iy * dy)'
        return ExpressionFunction('{} * {} * 1.'.format(X, Y),
                                  2, (), {}, {'ix': ix, 'iy': iy, 'dx': 1. / num_blocks[0], 'dy': 1. / num_blocks[1]},
                                  name='diffusion_{}_{}'.format(ix, iy))

    return StationaryProblem(

        domain=RectDomain(),

        rhs=ConstantFunction(dim_domain=2, value=1.),

        diffusion=LincombFunction([diffusion_function_factory(ix, iy)
                                   for ix, iy in product(range(num_blocks[0]), range(num_blocks[1]))],
                                  [parameter_functional_factory(ix, iy)
                                   for ix, iy in product(range(num_blocks[0]), range(num_blocks[1]))],
                                  name='diffusion'),

        parameter_space=CubicParameterSpace({'diffusion': (num_blocks[1], num_blocks[0])}, *parameter_range),

        name='ThermalBlock({})'.format(num_blocks)

    )

def thermal_block_fenics():

    # assemble system matrices - FEniCS code
    ########################################

    import dolfin as df

    mesh = df.UnitSquareMesh(GRID_INTERVALS, GRID_INTERVALS, 'crossed')
    V = df.FunctionSpace(mesh, 'Lagrange', FENICS_ORDER)
    u = df.TrialFunction(V)
    v = df.TestFunction(V)

    diffusion = df.Expression('(lower0 <= x[0]) * (open0 ? (x[0] < upper0) : (x[0] <= upper0)) *' +
                              '(lower1 <= x[1]) * (open1 ? (x[1] < upper1) : (x[1] <= upper1))',
                              lower0=0., upper0=0., open0=0,
                              lower1=0., upper1=0., open1=0,
                              element=df.FunctionSpace(mesh, 'DG', 0).ufl_element())

    def assemble_matrix(x, y, nx, ny):
        diffusion.user_parameters['lower0'] = x/nx
        diffusion.user_parameters['lower1'] = y/ny
        diffusion.user_parameters['upper0'] = (x + 1)/nx
        diffusion.user_parameters['upper1'] = (y + 1)/ny
        diffusion.user_parameters['open0'] = (x + 1 == nx)
        diffusion.user_parameters['open1'] = (y + 1 == ny)
        return df.assemble(df.inner(diffusion * df.nabla_grad(u), df.nabla_grad(v)) * df.dx)

    mats = [assemble_matrix(x, y, XBLOCKS, YBLOCKS)
            for x in range(XBLOCKS) for y in range(YBLOCKS)]
    mat0 = mats[0].copy()
    mat0.zero()
    h1_mat = df.assemble(df.inner(df.nabla_grad(u), df.nabla_grad(v)) * df.dx)

    f = df.Constant(1.) * v * df.dx
    F = df.assemble(f)

    bc = df.DirichletBC(V, 0., df.DomainBoundary())
    for m in mats:
        bc.zero(m)
    bc.apply(mat0)
    bc.apply(h1_mat)
    bc.apply(F)

    # wrap everything as a pyMOR discretization
    ###########################################

    # FEniCS wrappers
    from pymor.bindings.fenics import FenicsVectorSpace, FenicsMatrixOperator, FenicsVisualizer

    # define parameter functionals (same as in pymor.analyticalproblems.thermalblock)
    parameter_functionals = [ProjectionParameterFunctional(component_name='diffusion',
                                                           component_shape=(YBLOCKS, XBLOCKS),
                                                           coordinates=(YBLOCKS - y - 1, x))
                             for x in range(XBLOCKS) for y in range(YBLOCKS)]

    # wrap operators
    ops = [FenicsMatrixOperator(mat0, V, V)] + [FenicsMatrixOperator(m, V, V) for m in mats]
    op = LincombOperator(ops, [1.] + parameter_functionals)
    rhs = VectorOperator(FenicsVectorSpace(V).make_array([F]))
    h1_product = FenicsMatrixOperator(h1_mat, V, V, name='h1_0_semi')

    # build discretization
    visualizer = FenicsVisualizer(FenicsVectorSpace(V))
    parameter_space = CubicParameterSpace(op.parameter_type, 0.1, 1.)
    d = StationaryDiscretization(op, rhs, products={'h1_0_semi': h1_product},
                                 parameter_space=parameter_space,
                                 visualizer=visualizer)

    return d

def thermal_block_problem(num_blocks=(3, 3), parameter_range=(0.1, 1)):
    """Analytical description of a 2D 'thermal block' diffusion problem.

    The problem is to solve the elliptic equation ::

      - ∇ ⋅ [ d(x, μ) ∇ u(x, μ) ] = f(x, μ)

    on the domain [0,1]^2 with Dirichlet zero boundary values. The domain is
    partitioned into nx x ny blocks and the diffusion function d(x, μ) is
    constant on each such block (i,j) with value μ_ij. ::

           ----------------------------
           |        |        |        |
           |  μ_11  |  μ_12  |  μ_13  |
           |        |        |        |
           |---------------------------
           |        |        |        |
           |  μ_21  |  μ_22  |  μ_23  |
           |        |        |        |
           ----------------------------

    Parameters
    ----------
    num_blocks
        The tuple `(nx, ny)`
    parameter_range
        A tuple `(μ_min, μ_max)`. Each |Parameter| component μ_ij is allowed
        to lie in the interval [μ_min, μ_max].
    """

    def parameter_functional_factory(ix, iy):
        return ProjectionParameterFunctional(component_name='diffusion',
                                             component_shape=(num_blocks[1], num_blocks[0]),
                                             coordinates=(num_blocks[1] - iy - 1, ix),
                                             name='diffusion_{}_{}'.format(ix, iy))

    def diffusion_function_factory(ix, iy):
        if ix + 1 < num_blocks[0]:
            X = '(x[..., 0] >= ix * dx) * (x[..., 0] < (ix + 1) * dx)'
        else:
            X = '(x[..., 0] >= ix * dx)'
        if iy + 1 < num_blocks[1]:
            Y = '(x[..., 1] >= iy * dy) * (x[..., 1] < (iy + 1) * dy)'
        else:
            Y = '(x[..., 1] >= iy * dy)'
        return ExpressionFunction('{} * {} * 1.'.format(X, Y),
                                  2, (), {}, {'ix': ix, 'iy': iy, 'dx': 1. / num_blocks[0], 'dy': 1. / num_blocks[1]},
                                  name='diffusion_{}_{}'.format(ix, iy))

    problem = StationaryProblem(

        domain=RectDomain(),

        rhs=ConstantFunction(dim_domain=2, value=1.),

        diffusion=LincombFunction([diffusion_function_factory(ix, iy)
                                   for ix, iy in product(range(num_blocks[0]), range(num_blocks[1]))],
                                  [parameter_functional_factory(ix, iy)
                                   for ix, iy in product(range(num_blocks[0]), range(num_blocks[1]))],
                                  name='diffusion'),

        parameter_space=CubicParameterSpace({'diffusion': (num_blocks[1], num_blocks[0])}, *parameter_range),

        name='ThermalBlock({})'.format(num_blocks)
    )

    d, _ = discretize_stationary_cg(problem, diameter=1. / GRID_INTERVALS)
    return d

####################################################################################################
# Reduction algorithms                                                                             #
####################################################################################################


def reduce_naive(d, reductor, basis_size):

    training_set = d.parameter_space.sample_randomly(basis_size)

    for mu in training_set:
        u = d.solve(mu)
        reductor.extend_basis(u, 'trivial')

    rd = reductor.reduce()

    return rd


def reduce_greedy(d, reductor, snapshots, basis_size):

    training_set = d.parameter_space.sample_uniformly(snapshots)
    pool = new_parallel_pool()

    greedy_data = greedy(d, reductor, training_set,
                         extension_params={'method': 'gram_schmidt'},
                         max_extensions=basis_size,
                         pool=pool)

    return greedy_data['rd']


def reduce_adaptive_greedy(d, reductor, validation_mus, basis_size):

    pool = new_parallel_pool()

    greedy_data = adaptive_greedy(d, reductor, validation_mus=-validation_mus,
                                  extension_params={'method': 'gram_schmidt'},
                                  max_extensions=basis_size,
                                  pool=pool)

    return greedy_data['rd']


def reduce_pod(d, reductor, snapshots, basis_size):

    training_set = d.parameter_space.sample_uniformly(snapshots)

    snapshots = d.operator.source.empty()
    for mu in training_set:
        snapshots.append(d.solve(mu))

    basis, singular_values = pod(snapshots, modes=basis_size, product=reductor.product)
    reductor.extend_basis(basis, 'trivial')

    rd = reductor.reduce()

    return rd

# parameters for high-dimensional models
XBLOCKS = 2             # pyMOR/FEniCS
YBLOCKS = 2             # pyMOR/FEniCS
GRID_INTERVALS = 100    # pyMOR/FEniCS
FENICS_ORDER = 2
NGS_ORDER = 4
RBSIZE = 10
TEST = 5

# setup analytical problem
# discretize using continuous finite elements
# d  = thermal_block_problem(num_blocks=(XBLOCKS, YBLOCKS))
"""
d = thermal_block_fenics()
# select reduction algorithm with error estimator
coercivity_estimator = ExpressionParameterFunctional('min(diffusion)', d.parameter_type)
reductor = CoerciveRBReductor(d, product=d.h1_0_semi_product, coercivity_estimator=coercivity_estimator)

rd = reduce_naive(d, reductor, RBSIZE)

# evaluate the reduction error
results = reduction_error_analysis(rd, d=d, reductor=reductor, estimator=True,
                                   error_norms=[d.h1_0_semi_norm], condition=True,
                                   test_mus=TEST, random_seed=999, plot=True)

# show results
##############
print(results['summary'])
# import matplotlib.pyplot
# matplotlib.pyplot.show(results['figure'])

# visualize reduction error for worst-approximated mu
#####################################################
mumax = results['max_error_mus'][0, -1]
U = d.solve(mumax)
U_RB = reductor.reconstruct(rd.solve(mumax))
d.visualize((U, U_RB, U - U_RB), legend=('Detailed Solution', 'Reduced Solution', 'Error'), separate_colorbars=True, block=True)
"""

c = Capacitor()
c.space()


import unittest

from artap.problem import Problem
from artap.benchmark_functions import BinhAndKorn, AlpineFunction, Ackley
from artap.algorithm_genetic import NSGAII
from artap.results import Results


class BinhAndKornProblem(Problem):
    """
    Describe simple one objective optimization problem.
    Note: This test is not removed, because its not part of the cec2005 library.
    """

    def set(self):
        self.name = "TestNSGA2Optimization"
        self.parameters = [{'name': 'x_1', 'initial_value': 2.5, 'bounds': [0, 5]},
                           {'name': 'x_2', 'initial_value': 1.5, 'bounds': [0, 3]}]
        self.costs = [{'name': 'F_1', 'criteria': 'minimize'}, {'name': 'F_2', 'criteria': 'minimize'}]

    def evaluate(self, individual):
        function = BinhAndKorn()
        return function.eval(individual.vector)

    def evaluate_constraints(self, individual):
        return BinhAndKorn.constraints(individual.vector)


class TestNSGA2Optimization(unittest.TestCase):
    """ Tests simple one objective optimization problem."""

    def test_local_problem_nsga2(self):
        problem = BinhAndKornProblem()

        algorithm = NSGAII(problem)
        algorithm.options['max_population_number'] = 50
        algorithm.options['max_population_size'] = 200
        algorithm.options['calculate_gradients'] = True
        algorithm.options['verbose_level'] = 1

        algorithm.run()
        b = Results(problem)
        solution = b.pareto_values()
        wrong = 0

        for sol in solution:
            if abs(BinhAndKorn.approx(sol[0]) - sol[1]) > 0.1 * BinhAndKorn.approx(sol[0]) \
                    and 20 < sol[0] < 70:
                wrong += 1

        self.assertLessEqual(wrong, 5)

    def test_local_problem_nsga2(self, population_number=5):
        try:
            problem = BinhAndKornProblem()
            algorithm = NSGAII(problem)
            algorithm.options['max_population_number'] = population_number
            algorithm.options['max_population_size'] = 50
            algorithm.options['calculate_gradients'] = True
            algorithm.options['verbose_level'] = 1

            algorithm.run()
            b = Results(problem)
            solution = b.pareto_values()
            wrong = 0

            # c = GraphicalResults(problem)
            # c.plot_populations()

            for sol in solution:
                if abs(BinhAndKorn.approx(sol[0]) - sol[1]) > 0.1 * BinhAndKorn.approx(sol[0]) \
                        and 20 < sol[0] < 70:
                    wrong += 1

            self.assertLessEqual(wrong, 5)
        except AssertionError:
            # stochastic
            print("TestNSGA2Optimization::test_local_problem_nsga2", population_number)
            self.test_local_problem_nsga2(int(1.5 * population_number))


class TestAckley(unittest.TestCase):
    """ Tests that the nsga - ii can find the global optimum. """

    def test_local_problem(self, population_number=15):
        try:
            problem = Ackley(**{'dimension': 1})
            algorithm = NSGAII(problem)
            algorithm.options['max_population_number'] = population_number
            algorithm.options['max_population_size'] = 50
            algorithm.options['max_processes'] = 10
            algorithm.run()

            b = Results(problem)
            optimum = b.find_minimum('F_1')  # Takes last cost function
            self.assertAlmostEqual(optimum.costs[0], problem.global_optimum, 1)
        except AssertionError:
            # stochastic
            print("TestAckleyN222::test_local_problem", population_number)
            self.test_local_problem(int(1.5 * population_number))


class TestAlpine(unittest.TestCase):
    """ Tests that the nsga - ii can find the global optimum. """

    def test_local_problem(self, population_number=15):
        try:
            problem = AlpineFunction(**{'dimension': 3})
            algorithm = NSGAII(problem)
            algorithm.options['max_population_number'] = population_number
            algorithm.options['max_population_size'] = 100
            algorithm.options['max_processes'] = 10
            algorithm.run()

            b = Results(problem)
            optimum = b.find_minimum('F_1')  # Takes last cost function
            self.assertAlmostEqual(optimum.costs[0], problem.global_optimum, 1)
        except AssertionError:
            # stochastic
            print("TestAlpine::test_local_problem", population_number)
            self.test_local_problem(int(1.5 * population_number))


if __name__ == '__main__':
    unittest.main()



from artap.problem import Problem
from artap.algorithm_genetic import NSGAII
from artap.results import Results, GraphicalResults

import matplotlib.pyplot as plt


class GearDesignProblem(Problem):
    """
    Example from K.DEb Multi-objective evolutionary optimization problems, Wiley, 2001.
    pp 434.

    The objective of the task is to find the optimal turn numbers of a gearbox gear's.
    The gearbox contains for gears, the required gear ratio is 1/6.931, the number of
    the teeth must be integer numbers, all of these four variables must to be integers.

    The variable vector:    x = (x1, x2, x3, x4) = (Td, Tb, Ta, Tf)

    The problem can be formulated as a two variable optimization function:

    The first goal function's role is to minimize the  error between the obtained and
    the realized gear ratio:

    Minimize f1 = (1./6.931 - x1*x2/(x3*x4))**2.

    where x is the solution vector, contains the number of the teeths, these numbers
    are strictly integers

    Minimize f2 = max(x1, x2, x3, x4)

    subject to

            x1 e [12, 60] strictly integer
            x2 e [12, 60] strictly integer
            x3 e [12, 60] strictly integer
            x4 e [12, 60] strictly integer

    The two extremal solution from [DEB]:

    - solution E : x = (12, 12, 27, 37); Error f1 = 1.83e-8; Max teeth number = 37
    - solution D : x = (12, 12, 13, 13); Error f1 = 5.01e-1; Max teeth number = 13

    References:

    .: [DEB] Deb, K. (2001). Multi-objective optimization using evolutionary algorithms (Vol. 16). John Wiley & Sons.
    """
    def set(self):

        # Not mandatory to give a name for the test problem
        self.name = 'Gear Design'

        # Defines x_1 and x_2, which are the optimized parameters
        # and the bounds 'defines' the constraints of the optimization problem
        # nsga -- ii algorithm doesn't need an initial value for the definition
        self.parameters = [{'name':'x1', 'bounds': [12, 60], 'parameter_type':'integer'},
                           {'name':'x2', 'bounds': [12, 60], 'parameter_type':'integer'},
                           {'name':'x3', 'bounds': [12, 60], 'parameter_type':'integer'},
                           {'name':'x4', 'bounds': [12, 60], 'parameter_type':'integer'}]

        # The two, separate optimization functions and the direction of the optimization
        # is set to minimization. It is also possible to use the maximize keyword.
        self.costs = [{'name': 'f_1', 'criteria': 'minimize'},
                      {'name': 'f_2', 'criteria': 'minimize'}]

    def evaluate(self, x):
        f1 = (1./6.931 - (x.vector[0]*x.vector[1])/(x.vector[2]*x.vector[3]))**2.
        f2 = max(x.vector)
        return [f1, f2]

# Initialization of the problem
problem = GearDesignProblem()

# Perform the optimization iterating over 100 times on 100 individuals.
algorithm = NSGAII(problem)
algorithm.options['max_population_number'] = 100
algorithm.options['max_population_size'] = 100
algorithm.run()

# Post - processing the results
# reads in the result values into the b, results class
b = Results(problem)
# finding the pareto values
solution = b.pareto_values()

print(solution)
# Plotting out the resulting hyperbola with matplotlib
plt.scatter([s[0] for s in solution],
            [s[1] for s in solution])

#plt.xlim([12, 60])
#plt.ylim([0, 0.1])

plt.xlabel("$f_1(x)$ - Error")
plt.ylabel("$f_2(x)$ - Maximum size")

plt.show()


from .algorithm_genetic import GeneralEvolutionaryAlgorithm
from .job import Job
from .operators import GradientGeneration
from .population import Population
import time


class SweepAlgorithm(GeneralEvolutionaryAlgorithm):
    """
    Sweep Analysis
    """

    def __init__(self, problem, generator, name='SweepAlgorithm'):
        super().__init__(problem, name)

        self.generator = generator

    def run(self):
        t_s = time.time()
        # self.problem.logger.info("Sweep: {}: {} individuals".format(self.generator.__class__.__name__, self.population_size))

        # create initial population and evaluate individuals
        population = self.gen_initial_population()

        if self.options['calculate_gradients'] is True:
            # TODO: Move to Algorithm class here only call calculate_gradients(individuals)
            gradient_generator = GradientGeneration(self.problem.parameters)
            gradient_generator.init(population.individuals)
            gradient_population = Population(gradient_generator.generate())
            gradient_population.individuals = self.evaluate(gradient_population.individuals)
            population.individuals = self.evaluate_gradient(population.individuals, gradient_population.individuals)

        t = time.time() - t_s
        self.problem.logger.info("Sweep: elapsed time: {} s".format(t))



import unittest
from artap.individual import Individual
from artap.benchmark_functions import Rosenbrock, Ackley, Schwefel, Sphere, ModifiedEasom, Michaelwicz, Perm, Rastrigin, \
    SixHump, EqualityConstr, Griewank, Schubert, Zakharov, XinSheYang, XinSheYang2, XinSheYang3, Booth, GramacyLee, \
    AlpineFunction


class TestRosenbrock(unittest.TestCase):

    def test_rosenbrock2d(self):
        test2d = Rosenbrock(**{'dimension': 2})
        self.assertAlmostEqual(test2d.evaluate(Individual([1.0, 1.0]))[0], 0.0)

    def test_rosenbrock10d(self):
        test2d = Rosenbrock(**{'dimension': 10})
        self.assertAlmostEqual(test2d.evaluate(Individual([1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0]))[0], 0.0)


class TestAckley(unittest.TestCase):
    """ Tests for the synthetic benchmark functions """

    def test_ackley(self):
        test2d = Ackley(**{'dimension': 2})
        self.assertAlmostEqual(test2d.evaluate(Individual([0.0, 0.0]))[0], 0.0)

    def test_ackley10d(self):
        test2d = Ackley(**{'dimension': 10})
        self.assertAlmostEqual(test2d.evaluate(Individual([0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]))[0], 0.0)


class TestSphere(unittest.TestCase):

    def test_sphere(self):
        test7d = Sphere(**{'dimension': 7})
        self.assertAlmostEqual(test7d.evaluate(Individual(test7d.global_optimum_coords))[0], 0.0)


class TestSchwefel(unittest.TestCase):

    def test_schwefel(self):
        test2d = Schwefel(**{'dimension': 2})
        self.assertAlmostEqual(test2d.evaluate(Individual([420.9687, 420.9687]))[0], 0, 4)

        test10d = Schwefel(**{'dimension': 10})
        self.assertAlmostEqual(test2d.evaluate(Individual(test2d.global_optimum_coords))[0], 0, 4)


class TestModifiedEasom(unittest.TestCase):

    def test_measom(self):
        test2d = ModifiedEasom(**{'dimension': 2})
        self.assertAlmostEqual(test2d.evaluate(Individual(test2d.global_optimum_coords))[0], -1., 4)


class TestMichaelwicz(unittest.TestCase):

    def test_micha2d(self):
        test2d = Michaelwicz(**{'dimension': 2})
        self.assertAlmostEqual(test2d.evaluate(Individual(test2d.global_optimum_coords))[0], test2d.global_optimum, 3)


class TestPerm(unittest.TestCase):

    def test_perm(self):
        test2d = Perm(**{'dimension': 2})
        self.assertAlmostEqual(test2d.evaluate(Individual(test2d.global_optimum_coords))[0], test2d.global_optimum, 3)

    def test_perm10(self):
        test2d = Perm(**{'dimension': 10})
        self.assertAlmostEqual(test2d.evaluate(Individual(test2d.global_optimum_coords))[0], test2d.global_optimum, 3)


class TestRastrigin(unittest.TestCase):

    def test_rastrigin(self):
        test2d = Rastrigin(**{'dimension': 2})
        self.assertAlmostEqual(test2d.evaluate(Individual(test2d.global_optimum_coords))[0], test2d.global_optimum, 3)

    def test_rastrigin10(self):
        test2d = Rastrigin(**{'dimension': 10})
        self.assertAlmostEqual(test2d.evaluate(Individual(test2d.global_optimum_coords))[0], test2d.global_optimum, 3)


class TestSixhump(unittest.TestCase):
    # only 2d
    def test_sixhump(self):
        test2d = SixHump(**{'dimension': 2})
        self.assertAlmostEqual(test2d.evaluate(Individual(test2d.global_optimum_coords))[0], test2d.global_optimum, 3)


class TestEqualityConstr(unittest.TestCase):

    def test_equalityconstr(self):
        test2d = EqualityConstr(**{'dimension': 2})
        self.assertAlmostEqual(test2d.evaluate(Individual(test2d.global_optimum_coords))[0], test2d.global_optimum, 3)

    def test_equlaityconstr10(self):
        test2d = EqualityConstr(**{'dimension': 10})
        self.assertAlmostEqual(test2d.evaluate(Individual(test2d.global_optimum_coords))[0], test2d.global_optimum, 3)


class TestGriewank(unittest.TestCase):

    def test_griewank(self):
        test2d = Griewank(**{'dimension': 2})
        self.assertAlmostEqual(test2d.evaluate(Individual(test2d.global_optimum_coords))[0], test2d.global_optimum, 3)

    def test_griewank10(self):
        test2d = Griewank(**{'dimension': 10})
        self.assertAlmostEqual(test2d.evaluate(Individual(test2d.global_optimum_coords))[0], test2d.global_optimum, 3)


# class TestSchubert(unittest.TestCase):
# only 2d
# def test_schubert(self):
#     test2d = Schubert(**{'dimension': 2})
#     self.assertAlmostEqual(test2d.evaluate(test2d.global_optimum_coords), test2d.global_optimum, 3)

class TestZacharov(unittest.TestCase):

    def test_zacharov(self):
        test2d = Zakharov(**{'dimension': 2})
        self.assertAlmostEqual(test2d.evaluate(Individual(test2d.global_optimum_coords))[0], test2d.global_optimum, 3)

    def test_griewank10(self):
        test2d = Zakharov(**{'dimension': 10})
        self.assertAlmostEqual(test2d.evaluate(Individual(test2d.global_optimum_coords))[0], test2d.global_optimum, 3)


class TestXinSheYang(unittest.TestCase):

    def test_xinsheyang(self):
        test2d = XinSheYang(**{'dimension': 2})
        self.assertAlmostEqual(test2d.evaluate(Individual(test2d.global_optimum_coords))[0], test2d.global_optimum, 3)

    def test_xinsheyang10(self):
        test2d = XinSheYang(**{'dimension': 10})
        self.assertAlmostEqual(test2d.evaluate(Individual(test2d.global_optimum_coords))[0], test2d.global_optimum, 3)


class TestXinSheYang2(unittest.TestCase):

    def test_xinsheyang22(self):
        test2d = XinSheYang2(**{'dimension': 2})
        self.assertAlmostEqual(test2d.evaluate(Individual(test2d.global_optimum_coords))[0], test2d.global_optimum, 3)

    def test_xinsheyang210(self):
        test2d = XinSheYang2(**{'dimension': 10})
        self.assertAlmostEqual(test2d.evaluate(Individual(test2d.global_optimum_coords))[0], test2d.global_optimum, 3)


class TestXinSheYang3(unittest.TestCase):

    def test_xinsheyang32(self):
        test2d = XinSheYang3(**{'dimension': 2})
        self.assertAlmostEqual(test2d.evaluate(Individual(test2d.global_optimum_coords))[0], test2d.global_optimum, 3)

    def test_xinsheyang310(self):
        test2d = XinSheYang3(**{'dimension': 10})
        self.assertAlmostEqual(test2d.evaluate(Individual(test2d.global_optimum_coords))[0], test2d.global_optimum, 3)


class TestBooth(unittest.TestCase):

    def test_booth(self):
        test2d = Booth()
        self.assertAlmostEqual(test2d.evaluate(Individual(test2d.global_optimum_coords))[0], test2d.global_optimum, 3)


class TestGramacyLee(unittest.TestCase):

    def test_booth(self):
        test2d = GramacyLee()
        self.assertAlmostEqual(test2d.evaluate(Individual(test2d.global_optimum_coords))[0], test2d.global_optimum, 3)


class TestAlpine(unittest.TestCase):

    def test_alpine(self):
        test2d = AlpineFunction(**{'dimension': 10})
        self.assertAlmostEqual(test2d.evaluate(Individual(test2d.global_optimum_coords))[0], test2d.global_optimum, 3)


if __name__ == '__main__':
    unittest.main()



import unittest
import os

from artap.executor import LocalComsolExecutor
from artap.problem import Problem
from artap.algorithm import DummyAlgorithm
from artap.individual import Individual

__comsol__ = True
result = os.system('comsol --version')
if result != 0:
    __comsol__ = False


class ComsolProblem(Problem):
    """ Describe simple one objective optimization problem. """

    def set(self):
        self.name = "ComsolProblem"
        self.parameters = [{'name': 'a', 'initial_value': 10},
                           {'name': 'b', 'initial_value': 10}]
        self.costs = [{'name': 'F1', 'criteria': 'minimize'}]
        self.output_files = ["out.txt"]
        self.executor = LocalComsolExecutor(self,
                                            problem_file="./data/elstat.mph",
                                            output_files=self.output_files)

    def evaluate(self, individual):
        individual.dep_param = 0
        return self.executor.eval(individual)

    def parse_results(self, output_files, individual):
        output_file = output_files[0]
        path = output_file
        content = ""
        if os.path.exists(path):
            with open(path) as file:
                content = file.read()
        else:
            self.logger.error(
                "Output file '{}' doesn't exists.".format(path))

        lines = content.split("\n")
        line_with_results = lines[5]  # 5th line contains results
        result = float(line_with_results)
        return [result]


class TestLocalComsol(unittest.TestCase):
    @unittest.skipIf(__comsol__ is False, "require Comsol Multiphysics")
    def test_comsol_exec(self):
        """ Tests one calculation of goal function."""
        problem = ComsolProblem()

        individuals = [Individual([10, 10])]
        evaluator = DummyAlgorithm(problem)
        evaluator.evaluate(individuals)
        try:
            evaluator.evaluate(individuals)
            self.assertAlmostEqual(112.94090668383139, individuals[0].costs[0])
        except Exception as e:
            print("XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX")
            print(e)



import time
from abc import ABCMeta
from .individual import Individual


class Job(metaclass=ABCMeta):
    def __init__(self, problem, population=None):
        self.problem = problem
        self.population = population

    def evaluate(self, individual):
        # info
        individual.info["start_time"] = time.time()
        t_s = time.time()

        # set in progress
        individual.state = individual.State.IN_PROGRESS

        # check the constraints
        constraints = self.problem.evaluate_constraints(individual)

        if constraints:
            individual.feasible = sum(map(abs, constraints))

        # problem cost function evaluate only in that case when the problem is fits the constraints
        try:
            costs = self.problem.surrogate.evaluate(individual)
            if isinstance(costs, list):
                individual.costs = costs
            else:
                raise AssertionError("Costs type must be list.")

            # set evaluated
            individual.state = individual.State.EVALUATED

        except (TimeoutError, RuntimeError) as e:
            individual.state = Individual.State.FAILED

        # add to population
        if self.population is not None:
            self.population.individuals.append(individual)
            individual.info["population"] = self.problem.populations.index(self.population)

        # info
        individual.info["finish_time"] = time.time()

    def evaluate_scalar(self, x):
        # simple individual
        individual = Individual(x)
        self.evaluate(individual)

        return individual.costs[0]



#!/usr/bin/python3

from artap.problem import Problem
from artap.results import Results, GraphicalResults
from artap.datastore import FileDataStore

from shutil import copyfile

#database_name = "tmp/loudspeaker_multi_tmp.db"
#copyfile("tmp/loudspeaker_multi.db", database_name)
#database_name = "tmp/loudspeaker_single_tmp.db"
#copyfile("tmp/loudspeaker_single.db", database_name)

problem = Problem()
problem.data_store = FileDataStore(problem, database_name=database_name, mode="read")
results = Results(problem)

ind = 0
for population in problem.populations:
    ind = ind + len(population.individuals)

print("individuals = {}".format(ind))
print(results.parameters())
print(results.costs())

results = GraphicalResults(problem)
results.plot_scatter('F1', 'F2', filename="/tmp/scatter.pdf", population_number=None)
#results.plot_individuals('F', filename="/tmp/individuals.pdf")

"""
# CSV
import csv
with open('tmp/results.csv', 'w', newline='') as csvfile:
    spamwriter = csv.writer(csvfile, delimiter=';', quotechar='|', quoting=csv.QUOTE_MINIMAL)

    for population in problem.populations:
        for individual in population.individuals:
            spamwriter.writerow([individual.vector[0], individual.vector[1], individual.vector[2], individual.vector[3],
                                 individual.costs[0]])
"""


from abc import abstractmethod, ABC
import sys
import random
import math
import itertools
import numpy as np
from math import exp
from artap.individual import Individual
from artap.utils import VectorAndNumbers
from artap.doe import build_box_behnken, build_lhs, build_full_fact, build_plackett_burman
from copy import copy

EPSILON = sys.float_info.epsilon


class Operation(ABC):

    def __init__(self):
        pass

    @staticmethod
    def clip(value, min_value, max_value):
        return max(min_value, min(value, max_value))


class Generation(Operation):

    def __init__(self, parameters=None, individual_class=Individual):
        super().__init__()
        self.parameters = parameters
        self.individual_class = individual_class

    def create_individual(self, vector: list = []):
        return self.individual_class(vector)

    @abstractmethod
    def generate(self):
        pass


class CustomGeneration(Generation):

    def __init__(self, parameters=None, individual_class=Individual):
        super().__init__(parameters, individual_class)
        self.vectors = []

    def init(self, vectors):
        self.vectors = vectors

    def generate(self):
        individuals = []
        for vector in self.vectors:
            individuals.append(self.create_individual(vector))
        return individuals


class RandomGeneration(Generation):

    def __init__(self, parameters=None, individual_class=Individual):
        super().__init__(parameters, individual_class)
        self.number = 0

    def init(self, number):
        self.number = number

    def generate(self):
        individuals = []
        for i in range(self.number):
            vector = VectorAndNumbers.gen_vector(self.parameters)
            individuals.append(self.create_individual(vector))
        return individuals


class FullFactorGeneration(Generation):
    """
    Create a general full-factorial design
    Number of experiments (2 ** len(parameters) - without center, 3 ** len(parameters - with center)
    """

    def __init__(self, parameters=None, individual_class=Individual):
        super().__init__(parameters, individual_class)
        self.center = False

    def init(self, center):
        self.center = center

    def generate(self):
        dict_vars = {}
        for parameter in self.parameters:
            name = parameter['name']
            l_b = parameter['bounds'][0]
            u_b = parameter['bounds'][1]

            if self.center:
                dict_vars[name] = [l_b, (l_b + u_b) / 2.0, u_b]
            else:
                dict_vars[name] = [l_b, u_b]

        df = build_full_fact(dict_vars)

        individuals = []
        for vector in df:
            individuals.append(self.create_individual(vector))
        return individuals


class PlackettBurmanGeneration(Generation):
    """
    Create a general full-factorial design
    Number of experiments (2 ** len(parameters) - without center, 3 ** len(parameters - with center)
    """

    def __init__(self, parameters=None, individual_class=Individual):
        super().__init__(parameters, individual_class)

    def generate(self):
        dict_vars = {}
        for parameter in self.parameters:
            name = parameter['name']
            l_b = parameter['bounds'][0]
            u_b = parameter['bounds'][1]
            dict_vars[name] = [l_b, u_b]

        df = build_plackett_burman(dict_vars)
        # print(df)

        individuals = []
        for vector in df:
            individuals.append(self.create_individual(vector))
        return individuals


class BoxBehnkenGeneration(Generation):
    """
    Create a general full-factorial design
    # 3 params = 13 experiments
    # 4 params = 25 experiments
    # 5 params = 41 experiments
    # 6 params = 49 experiments
    # 7 params = 57 experiments
    # 8 params = 113 experiments
    https://en.wikipedia.org/wiki/Box%E2%80%93Behnken_design
    """

    def __init__(self, parameters=None, individual_class=Individual):
        super().__init__(parameters, individual_class, )

    def generate(self):
        dict_vars = {}
        for parameter in self.parameters:
            name = parameter['name']
            l_b = parameter['bounds'][0]
            u_b = parameter['bounds'][1]
            dict_vars[name] = [l_b, u_b]

        df = build_box_behnken(dict_vars)
        # print(df)

        individuals = []
        for vector in df:
            individuals.append(self.create_individual(vector))
        return individuals


class LHSGeneration(Generation):
    """
    Builds a Latin Hypercube design dataframe from a dictionary of factor/level ranges.
    """

    def __init__(self, parameters=None, individual_class=Individual):
        super().__init__(parameters, individual_class)
        self.number = 0

    def init(self, number):
        self.number = number

    def generate(self):
        dict_vars = {}
        for parameter in self.parameters:
            name = parameter["name"]
            l_b = parameter['bounds'][0]
            u_b = parameter['bounds'][1]
            dict_vars[name] = [l_b, u_b]

        df = build_lhs(dict_vars, num_samples=self.number)
        # print(df)

        individuals = []
        for vector in df:
            individuals.append(self.create_individual(vector))
        return individuals


class GradientGeneration(Generation):

    def __init__(self, parameters=None, individual_class=Individual):
        super().__init__(parameters, individual_class)
        self.delta = 1e-6
        self.individuals = None

    def init(self, individuals):
        self.individuals = individuals

    def generate(self):
        new_individuals = []
        k = 0
        for individual in self.individuals:
            for i in range(len(individual.vector)):
                vector = individual.vector.copy()
                vector[i] -= self.delta
                new_individuals.append(self.create_individual(vector))
                new_individuals[-1].depends_on = k
                new_individuals[-1].modified_param = i
            k += 1
        return new_individuals


class Mutation(Operation):

    def __init__(self, parameters, probability):
        super().__init__()
        self.parameters = parameters
        self.probability = probability

    @abstractmethod
    def mutate(self, p):
        pass


class SimpleMutation(Mutation):
    def __init__(self, parameters, probability):
        super().__init__(parameters, probability)

    def mutate(self, p):
        """ uniform random mutation """
        mutation_space = 0.1
        vector = []

        for i, parameter in enumerate(self.parameters):
            if random.uniform(0, 1) < self.probability:
                l_b = parameter['bounds'][0]
                u_b = parameter['bounds'][1]

                para_range = mutation_space * (u_b - l_b)
                mutation = random.uniform(-para_range, para_range)
                # vector.append(self.clip(p.vector[i] + mutation, l_b, u_b))
                vector.append(p.vector[i] + mutation)
            else:
                vector.append(p.vector[i])

        p_new = p.__class__(vector)
        return p_new


class PmMutation(Mutation):
    """
    PmMutation -- for nsga2 and epsMoEA

    This operator can handle real, integer and boolean optimization parameters.
    The class contains two-kind of operators as the original (Deb's [...]) implementation .

    This is a difference between Artap and Platypus, where the Integer numbers are encoded (Gray-encoding) and handled
    as binary numbers with the Bitflip operator.
    """

    def __init__(self, parameters, probability, distribution_index=20):
        super().__init__(parameters, probability)
        self.distribution_index = distribution_index

    def mutate(self, parent):
        vector = []

        for i, parameter in enumerate(self.parameters):
            if isinstance(parent.vector[i], list):
                vector.append(self.bitflip(parent.vector[i]))

            else:
                if random.uniform(0, 1) < self.probability:
                    l_b = parameter['bounds'][0]
                    u_b = parameter['bounds'][1]

                    if isinstance(parent.vector[i], float):
                        vector.append(self.pm_mutation(parent.vector[i], l_b, u_b))

                    if isinstance(parent.vector[i], int):
                        vector.append(int(self.pm_mutation(parent.vector[i], l_b, u_b)))
                else:
                    vector.append(parent.vector[i])

        p_new = parent.__class__(vector)
        return p_new

    def pm_mutation(self, x, lb, ub):
        """
        Polynomial mutation for float and integer parameters.

        :param x: represents one parameter of the problem
        :param lb: lower bound
        :param ub: upper bound
        :return:
        """
        u = random.uniform(0, 1)
        dx = ub - lb

        if u < 0.5:
            bl = (x - lb) / dx
            b = 2.0 * u + (1.0 - 2.0 * u) * pow(1.0 - bl, self.distribution_index + 1.0)
            delta = pow(b, 1.0 / (self.distribution_index + 1.0)) - 1.0
        else:
            bu = (ub - x) / dx
            b = 2.0 * (1.0 - u) + 2.0 * (u - 0.5) * pow(1.0 - bu, self.distribution_index + 1.0)
            delta = 1.0 - pow(b, 1.0 / (self.distribution_index + 1.0))

        x = x + delta * dx
        x = self.clip(x, lb, ub)

        return x

    def bitflip(self, x: list):

        for j in range(1, len(x)):
            if random.uniform(0.0, 1.0) <= self.probability:
                x[j] = not x[j]

        return x


class SwarmMutation(Mutation):
    """
    This swarm mutator operator is made for the original PSO algorithm, which defined by Kennedy and Eberhart in 1995

    PSO shares many similarities with evolutionary computation. Both algorithms start with a group of a randomly
    generated population. Both update the population iteratively and search for the optimum with stochastic techniques.
    The main difference between them is in the information sharing mechanism. In EA, only the individuals of current
    generation share information with each other, and any individual has a chance to give out information to others.
    In PSO, actually not the current individuals share information with each other, but the individuals of previous
    generation (the optimal particles) give out information to the current ones. In other words, the information sharing
    is one-way in PSO.
    """

    def __init__(self, parameters, probability=1):
        super().__init__(parameters, probability)
        self.w = 0.1  # constant inertia weight (how much to weigh the previous velocity)
        self.c1 = 2.  # cognitive constant
        self.c2 = 1.  # social constant
        self.best_individual = None

    def evaluate_best_individual(self, individual):
        """ Determines the best individual in the swarm """
        dominates = True

        for i in range(len(individual.best_costs)):
            if individual.costs[i] > individual.best_costs[i]:
                dominates = False

        # check to see if the current position is an individual best
        if dominates:
            individual.best_vector = individual.vector
            individual.best_costs = individual.costs

    # update new particle velocity
    def update_velocity(self, individual):

        for i in range(0, len(individual.vector)):
            r1 = 0.1 * random.random()
            r2 = 0.1 * random.random()

            vel_cognitive = self.c1 * r1 * (individual.best_vector[i] - individual.vector[i])
            vel_social = self.c2 * r2 * (self.best_individual.vector[i] - individual.vector[i])
            individual.velocity_i[i] = self.w * individual.velocity_i[i] + vel_cognitive + vel_social

    # update the particle position based off new velocity updates
    def update_position(self, individual):

        for parameter, i in zip(self.parameters, range(len(individual.vector))):
            individual.vector[i] = individual.vector[i] + individual.velocity_i[i]

            # adjust maximum position if necessary
            if individual.vector[i] > parameter['bounds'][1]:
                individual.vector[i] = parameter['bounds'][1]

            # adjust minimum position if necessary
            if individual.vector[i] < parameter['bounds'][0]:
                individual.vector[i] = parameter['bounds'][0]

    def update(self, best_individual):
        self.best_individual = best_individual

    def mutate(self, p):
        self.update_velocity(p)
        self.update_position(p)
        return p


class SwarmMutationTVIW(SwarmMutation):
    """
    This is an improvement of the original PSO algorithm with Time Varying Inertia Weight operators.

    Empirical study of particle swarm optimization,” in Proc. IEEE Int. Congr. Evolutionary Computation, vol. 3,
    1999, pp. 101–106.

    Shi and Eberhart have observed that the optimal solution can be improved by varying the inertia weight value from
    0.9 (at the beginning of the search) to 0.4 (at the end of the search) for most problems. This modification to the
    original PSO concept has been considered as the basis for two novel strategies introduced in this paper. Hereafter,
    in this paper, this version of PSO is referred to as time-varying inertia weight factor method

    Contras:
    -------
    - PSO-TVIW concept is not very effective for tracking dynamic systems

    - its ability to fine tune the optimum solution is comparatively weak, mainly due
      to the lack of diversity at the end of the search

    R. C. Eberhart and Y. Shi, “Tracking and optimizing dynamic systems with particle swarms,” in Proc. IEEE Congr.
    Evolutionary Computation 2001, Seoul, Korea, 2001, pp. 94–97
    """

    def __init__(self, parameters, probability=1, nr_maxgen=100):
        super().__init__(parameters, probability)
        self.w1 = 0.9  # inertia weight is calculated from w1 and w2
        self.w2 = 0.4
        self.c1 = 2.  # cognitive constant
        self.c2 = 1.  # social constant
        self.best_individual = None

        # new parameters
        self.max_nr_generations = nr_maxgen
        self.current_iter = 0.

    # update new particle velocity
    def update_velocity(self, individual):
        """
        :param nr_generations: total number of generations, during the calculation, MAXITER
        :param iteration_nr: actual generation
        """
        for i in range(0, len(individual.vector)):
            r1 = 0.1 * random.random()
            r2 = 0.1 * random.random()

            # (w1-w2)*(MAX_ITER-iter)/MAX_ITER
            w = (self.w1 - self.w2) * (self.max_nr_generations - self.current_iter) / self.max_nr_generations + self.w2

            vel_cognitive = self.c1 * r1 * (individual.best_vector[i] - individual.vector[i])
            vel_social = self.c2 * r2 * (self.best_individual.vector[i] - individual.vector[i])
            individual.velocity_i[i] = w * individual.velocity_i[i] + vel_cognitive + vel_social

            self.current_iter += 1.


class SwarmMutationRandIW(SwarmMutation):
    """
    In this variation, the inertia weght is changing randomly,the mean value of the inertia weight is 0.75.
    This modification was inspired by Clerc’s constriction factor concept,  in which the inertia weight is
    kept constant at 0.729 and both acceleration coefficients are kept constant at 1.494.
    Therefore, when random inertia weight factor method is used the acceleration coefficients are kept constant at 1.494.

    Contras:
    -------
    """

    def __init__(self, parameters, probability=1):
        super().__init__(parameters, probability)
        self.w = 0.5  # inertia weight -> changed randomly
        self.c1 = 2.  # cognitive constant
        self.c2 = 1.  # social constant
        self.best_individual = None

    # update new particle velocity
    def update_velocity(self, individual):
        """
        :param nr_generations: total number of generations, during the calculation, MAXITER
        :param iteration_nr: actual generation
        """
        for i in range(0, len(individual.vector)):
            r1 = 0.1 * random.random()
            r2 = 0.1 * random.random()

            # (w1-w2)*(MAX_ITER-iter)/MAX_ITER
            w = self.w * random.random() / 2.

            vel_cognitive = self.c1 * r1 * (individual.best_vector[i] - individual.vector[i])
            vel_social = self.c2 * r2 * (self.best_individual.vector[i] - individual.vector[i])
            individual.velocity_i[i] = w * individual.velocity_i[i] + vel_cognitive + vel_social


class FireflyMutation(SwarmMutation):
    """
    Firefly algorithm is a modification of the original pso algorithms. The idea is that it mimics the behaviour of
    the fireflies, which uses specfic light combinations for hunting and dating. This algorithm mimics the dating
    behaviour of these bugs. The algorithm is originally published by [1]

    The brightest individual attracts the darkest ones, this starts to go to that direction, if there is not an
    existing brighter solutions, the algorithm randomly steps one into another direction.

    This operator makes the following
    ---------------------------------

    - calculates the distance between two points, because its correlates with that value.
    - the brightness of the other point
    - the mutator constant, which contains a damping factor [2]?

    The k+1 th position of the j(th) individual is calculated by the following formula
    ---------------------------------------------------------------------------

        b0=2;               # Attraction Coefficient Base Value
        a=0.2;              # Mutation Coefficient
        ad=0.98;            # Mutation Coefficient Damping Ratio -- decreases the initial value of a after each
                              iteration step
        gamma = 1.          # light absorbtion coefficient

        x(j,k+1) = x(j,k) + b0*exp(-gamma*r^2) + sum_{i<j}[x(j,k) - x(i,k)] + a*(rand[0,1] - 0.5)

    [1] Yang, Xin-She. "Firefly algorithms for multimodal optimization." International symposium on stochastic algorithms.
        Springer, Berlin, Heidelberg, 2009.
    [2] firefly algorithm implementation from www.Yarpiz.com

    [3] https://nl.mathworks.com/matlabcentral/fileexchange/29693-firefly-algorithm

     Similarly, alpha should also be linked with scales, the steps should not too large or too small, often steps
     are about 1/10 to 1/100 of the domain size. In addition, alpha should be reduced gradually using
     alpha=alpha_0 delta^t during eteration t.  Typically, delta=0.9 to 0.99 will be a good choice.
    """

    def __init__(self, parameters, probability=0.05):
        super().__init__(parameters, probability)

        self.beta = 2.0
        self.alpha = 0.2
        self.ad = 0.98
        self.gamma = 1.

        self.best_individual = None

    def dominate(self, current, other):
        """True if other dominates over current """
        dominates = True

        for i in range(len(current.costs)):
            if other.signed_costs[i] > current.signed_costs[i]:
                dominates = False

        return dominates

    def update_coefficient_a(self):
        """ Updates the mutation coefficient with the damping factor, after each iteration step """
        self.alpha *= self.ad
        return

    # update new particle velocity
    def update_velocity_ij(self, current, other):
        """
        This algorithm has a two-layered hierarchy, because every individual calculates an approximative next position
        from the light intensity between two selected points.
        """
        r2 = 0.  # euclidean distance

        for i, param in enumerate(self.parameters):

            lb = param['bounds'][0]
            ub = param['bounds'][1]

            # elementary distance of the particle
            e = self.probability * (ub - lb)
            # distance between the two individuals
            r2 += current.vector[i] ** 2. + other.vector[i] ** 2.
            vel_attraction = self.beta * exp(-self.gamma * r2 ** 2.)

            v_rd = self.alpha * (random.random() - 0.5) * e
            current.velocity_i[i] = vel_attraction + v_rd

            # for individual in offsprings:
            #     for other in offsprings:
            #         dominates = False
            #
            #         if individual is not other:
            #             dominates = True
            #
            #         for i in range(len(individual.signed_costs)):
            #             if individual.signed_costs[i] > offsprings.signed_costs[i]:
            #                 dominates = False
            #
            #         if dominates:
            #
            #             r2 = 0
            #             for i, param in enumerate(self.parameters):
            #                 l_b = param['bounds'][0]
            #                 u_b = param['bounds'][1]
            #
            #                 # elementary distance of the particle (to do not )
            #                 e = self.probability*(u_b-l_b)
            #                 # distance between the two individuals
            #                 r2 += individual.vector[i]**2. + other.vector[i]**2.
            #
            #                 vel_attraction = self.beta*exp(-self.gamma*r2**2.)
            #
            #                 v_rd = self.alpha * (random.random() - 0.5) * e
            #                 individual.velocity_i[i] = vel_attraction + v_rd
            #
            #                 temp = copy(individual)
            #                 self.evaluate(temp)
            #
            #                 dom = True
            #                 for j in range(len(individual.best_costs)):
            #                     if individual.signed_costs[j] > offsprings.signed_costs[j]:
            #                         dom = False
            #
            #                 if dom is True:
            #                     individual = copy(temp)

            return

    def mutate_ij(self, p, q):
        """
        For firefly algorithm, because of the two-layered hierarchy.

        :param p:
        :param q:
        :return:
        """
        self.update_velocity_ij(p,q)
        self.update_position(p)
        return


class SwarmMutationTVAC(SwarmMutation):
    """
    Time-varying acceleration coefficients as a new parameter automation strategy for the PSO concept.

    An improved optimum solution for most of the benchmarks was observed when changing c1 from 2.5 to 0.5
    and changing c2 from 0.5 to 2.5, over the full range of the search.
    Therefore, these values are used for the rest of the work. With this modification, a significant improvement of
    the optimum value and the rate of convergence were observed, particularly for unimodal functions, compared with
    the PSO-TVIW. However, it has been observed that the performance of the PSO-TVAC method is similar or poor
    for multimodal functions. In contrast, compared with the PSO-RANDIW method an improved performance has been
    observed with the PSO-TVAC for multimodal functions.
    However, for unimodal functions, the PSO-RANDIW method showed significantly quick convergence to a good solution
    compared with the PSO-TVAC method. The results are presented and discussed in Section V.

    Pros: improved convergence rate in case of multi-modal functions

    Cons: significantly slower convergence rate than PSO-RandIW for unimodal functions
    """

    def __init__(self, parameters, probability=1):
        super().__init__(parameters, probability)
        self.w = 0.9  # inertia weight
        self.c1i = 0.5  # cognitive constant initial value
        self.c1f = 2.5  # cognitive constant final value
        self.c2i = 2.5  # social constant initial value
        self.c2f = 0.5  # social constant final value
        self.best_individual = None

    # update new particle velocity
    def update_velocity(self, individual, nr_generations, iteration_nr):
        """
        :param nr_generations: total number of generations, during the calculation, MAXITER
        :param iteration_nr: actual generation
        """
        for i in range(0, len(individual.vector)):
            r1 = 0.1 * random.random()
            r2 = 0.1 * random.random()

            # (c1f-c1i)*(MAX_ITER-iter)/MAX_ITER +
            c1 = (self.c1f - self.c1i) * (nr_generations - iteration_nr) / nr_generations + self.c1i
            c2 = (self.c2f - self.c2i) * (nr_generations - iteration_nr) / nr_generations + self.c2i

            vel_cognitive = c1 * r1 * (individual.best_vector[i] - individual.vector[i])
            vel_social = c2 * r2 * (self.best_individual.vector[i] - individual.vector[i])
            individual.velocity_i[i] = self.w * individual.velocity_i[i] + vel_cognitive + vel_social


class Selection(Operation):

    def __init__(self, parameters, part_num=2):
        super().__init__()
        self.parameters = parameters
        # self.signs = signs
        self.part_num = part_num
        self.comparator = ParetoDominance()

    @abstractmethod
    def select(self, population):
        pass

    def is_dominate(self, p, q):
        """
        :param p: current solution
        :param q: candidate
        :return: True if the candidate is better than the current solution
        """
        dominate = False

        # The cost function can be a float or a list of floats
        for i in range(0, len(p.costs)):
            if p.costs[i] > q.costs[i]:
                return False
            if p.costs[i] <= q.costs[i]:
                dominate = True
        return dominate

    def non_dominated_sort(self, population):
        pareto_front = []
        front_number = 1

        for p in population:
            p.domination_counter = 0
            p.front_number = None
            p.dominate = set()

            for q in population:
                if p is q:
                    continue
                if self.comparator.compare(p, q) == 1:  # TODO: simplify
                    p.dominate.add(q)
                elif self.comparator.compare(q, p) == 1:
                    p.domination_counter = p.domination_counter + 1

            if p.domination_counter == 0:
                p.front_number = front_number
                pareto_front.append(p)

        while not len(pareto_front) == 0:
            front_number += 1
            temp_set = []
            for p in pareto_front:
                for q in p.dominate:
                    q.domination_counter -= 1
                    if q.domination_counter == 0 and q.front_number is None:
                        q.front_number = front_number
                        temp_set.append(q)
            pareto_front = temp_set

    def sort_by_coordinate(self, population, dim):
        population.sort(key=lambda x: x.costs[dim])
        return population

    def crowding_distance(self, population):
        infinite = float("inf")
        n = len(population[0].costs)
        for dim in range(0, n):
            new_list = self.sort_by_coordinate(population, dim)

            new_list[0].crowding_distance += infinite
            new_list[-1].crowding_distance += infinite
            max_distance = new_list[0].vector[dim] - new_list[-1].vector[dim]
            for i in range(1, len(new_list) - 1):
                distance = new_list[i - 1].vector[dim] - new_list[i + 1].vector[dim]
                if max_distance == 0:
                    new_list[i].crowding_distance = 0
                else:
                    new_list[i].crowding_distance += distance / max_distance

        for p in population:
            p.crowding_distance = p.crowding_distance / n


class DummySelection(Selection):

    def __init__(self, parameters, part_num=2):
        super().__init__(parameters, part_num)

    def select(self, individuals):
        selection = []
        for individual in individuals:
            candidate = individual.__class__(individual.vector)
            candidate.costs = individual.costs
            candidate.front_number = individual.front_number
            candidate.best_vector = individual.best_vector
            candidate.best_costs = individual.best_costs

            selection.append(candidate)

        return selection


class Dominance(ABC):
    def __init__(self):
        pass

    def compare(self, p, q):
        raise NotImplementedError("method not implemented")


class EpsilonDominance(Dominance):
    """
    Epsilon dominance.

        Similar to Pareto dominance except if the two solutions are contained
        within the same epsilon-box, the solution closer to the optimal corner
        or the box is preferred.
    """

    def __init__(self, epsilons):
        super(EpsilonDominance, self).__init__()

        if hasattr(epsilons, "__getitem__"):
            self.epsilons = epsilons
        else:
            self.epsilons = [epsilons]

    def same_box(self, p, q):
        # first check constraint violation
        if p.feasible != q.feasible:
            if p.feasible == 0:
                return False
            elif q.feasible == 0:
                return False
            elif p.feasible < q.feasible:
                return False
            elif q.feasible < p.feasible:
                return False

        # then use epsilon dominance on the objectives
        dominate1 = False
        dominate2 = False

        for i in range(len(p.costs)):
            o1 = p.costs[i]
            o2 = q.costs[i]

            # in artap we cannot change the direction of the optimization in this way
            # if problem.directions[i] == Problem.MAXIMIZE:
            #    o1 = -o1
            #    o2 = -o2

            epsilon = float(self.epsilons[i % len(self.epsilons)])
            i1 = math.floor(o1 / epsilon)
            i2 = math.floor(o2 / epsilon)

            if i1 < i2:
                dominate1 = True
                if dominate2:
                    return False
            elif i1 > i2:
                dominate2 = True
                if dominate1:
                    return False

        if not dominate1 and not dominate2:
            return True
        else:
            return False

    def compare(self, p: list, q: list):

        # first check constraint violation, the last item is the feasibility, which is a real number if its zero,
        # it means that the solution is feasible
        if p[-1] != q[-1]:
            if p[-1] == 0:
                return 1 # p dominates
            elif q[-1] == 0:
                return 2 # q is dominates
            elif p[-1] < q[-1]:
                return 1 # p is dominates
            elif q[-1] < p[-1]:
                return 2 # q is dominates

        # then use epsilon dominance on the objectives
        dominate_p = False
        dominate_q = False

        for i,(p_costs, q_costs) in enumerate(p[:-1], q[:-1]):

            epsilon = float(self.epsilons[i % len(self.epsilons)])

            pepsi = math.floor(p_costs / epsilon)
            qepsi = math.floor(q_costs / epsilon)

            if pepsi > qepsi:
                dominate_q = True
                if dominate_p:
                    return 0
            else:
                dominate_p = True
                if dominate_q:
                    return 0


        if not dominate_p and not dominate_q:
            dist1 = 0.0
            dist2 = 0.0

            for i in range(len(p.costs)):
                o1 = p.costs[i]
                o2 = q.costs[i]

                epsilon = float(self.epsilons[i % len(self.epsilons)])
                i1 = math.floor(o1 / epsilon)
                i2 = math.floor(o2 / epsilon)

                dist1 += math.pow(o1 - i1 * epsilon, 2.0)
                dist2 += math.pow(o2 - i2 * epsilon, 2.0)

            if dist1 < dist2:
                return 1
            else:
                return 2
        elif dominate_p:
            return 1
        else:
            return 2


class ParetoDominance(Dominance):
    """Pareto dominance with constraints.

    If either solution violates constraints, then the solution with a smaller
    constraint violation is preferred.  If both solutions are feasible, then
    Pareto dominance is used to select the preferred solution.
    """

    def __init__(self, epsilons=None):
        super(ParetoDominance, self).__init__()
        # self.signs = signs deprecated

    def compare(self, p: list, q: list):
        """
        Here, p and q are tuples, which contains the (feasibility index, cost vector)
        """
        # first check constraint violation, the feasibility is written to the last place in the list
        if p[-1] != q[-1]:
            if p[-1] == 0:
                return 1
            elif q[-1] == 0:
                return 2
            elif p[-1] < q[-1]:
                return 1
            elif q[-1] < p[-1]:
                return 2

        dominate_p = False
        dominate_q = False

        for (p_costs, q_costs) in zip(p[:-1], q[:-1]):

            if p_costs > q_costs:
                dominate_q = True
                if dominate_p:
                    return 0
            else:
                dominate_p = True
                if dominate_q:
                    return 0

        if dominate_q == dominate_p:
            return 0
        elif dominate_p:
            return 1
        else:
            return 2


class Selection(Operation):

    def __init__(self, parameters, sign=None, part_num=2, dominance=ParetoDominance):
        """

        :param parameters:
        :param sign: one value from now, because its goal is to tell the direction of the optimum min or max
        :param part_num:
        :param dominance:
        """
        super().__init__()
        self.parameters = parameters
        self.part_num = part_num
        self.comparator = dominance()  # ParetoDominance()
        self.signs = sign

    @abstractmethod
    def select(self, population):
        pass

    def is_dominate(self, p, q):
        """
        :param p: current solution
        :param q: candidate
        :return: True if the candidate is better than the current solution
        """
        # The cost function can be a float or a list of floats
        for p_costs, q_costs in zip(p.costs, q.costs):
            if p_costs > q_costs:
                return False
            else:
                return True

    def sorting(self, generation):
        """
        This shoring can be dominated or non-dominated.
        :param generation: means the list of the population, it can be list of the individuals or the archive.
        :return:
        """
        pareto_front = []
        front_number = 1

        for p in generation:
            p.domination_counter = 0
            p.front_number = None
            p.dominate = set()

            for q in generation:
                if p is q:
                    continue
                if self.comparator.compare(p.signed_costs, q.signed_costs) == 1:  # TODO: simplify
                    p.dominate.add(q)
                elif self.comparator.compare(q.signed_costs, p.signed_costs) == 1:
                    p.domination_counter += 1

            if p.domination_counter == 0:
                p.front_number = front_number
                pareto_front.append(p)

        while not len(pareto_front) == 0:
            front_number += 1
            temp_set = []
            for p in pareto_front:
                for q in p.dominate:
                    q.domination_counter -= 1
                    if q.domination_counter == 0 and q.front_number is None:
                        q.front_number = front_number
                        temp_set.append(q)
            pareto_front = temp_set
        return

    def sort_by_coordinate(self, population, dim):
        population.sort(key=lambda x: x.costs[dim])
        return population

    def crowding_distance(self, population):
        infinite = float("inf")
        n = len(population[0].costs)
        for dim in range(0, n):
            new_list = self.sort_by_coordinate(population, dim)

            new_list[0].crowding_distance += infinite
            new_list[-1].crowding_distance += infinite
            max_distance = new_list[0].vector[dim] - new_list[-1].vector[dim]
            for i in range(1, len(new_list) - 1):
                distance = new_list[i - 1].vector[dim] - new_list[i + 1].vector[dim]
                if max_distance == 0:
                    new_list[i].crowding_distance = 0
                else:
                    new_list[i].crowding_distance += distance / max_distance

        for p in population:
            p.crowding_distance = p.crowding_distance / n


class DummySelection(Selection):

    def __init__(self, parameters, sign, part_num=2):
        super().__init__(parameters, sign, part_num)

    def select(self, individuals):
        selection = []
        for individual in individuals:
            candidate = individual.__class__(individual.vector)
            candidate.costs = individual.costs
            candidate.front_number = individual.front_number
            candidate.best_vector = individual.best_vector
            candidate.best_costs = individual.best_costs

            selection.append(candidate)
        return selection


class TournamentSelection(Selection):

    def __init__(self, parameters, dominance=ParetoDominance, epsilons=None):
        super().__init__(parameters)
        self.dominance = dominance(epsilons=epsilons)
        # self.signs = sign
        # self.dominance = EpsilonDominance([0.01])

    def select(self, population):
        """
        Binary tournament selection:
        An individual is selected in the rank is lesser than the other or if crowding distance is greater than the other
        """

        # participants = random.sample(population, 2)
        # return min(participants, key=lambda x: (x.front_number, -x.crowding_distance))

        winner = random.choice(population)

        for _ in range(self.part_num - 1):
            candidate = random.choice(population)

            flag = self.dominance.compare(winner.signed_costs, candidate.signed_costs)

            if flag > 0:
                winner = candidate

        return winner

        # participants = random.sample(parents, part_num)
        # return min(participants, key=lambda x: (x.front_number, -x.crowding_distance))


class Archive(object):
    """An archive only containing non-dominated solutions."""

    def __init__(self, signs, dominance=ParetoDominance):
        super(Archive, self).__init__()
        self._dominance = dominance(signs=signs)
        self._contents = []

    def add(self, solution):
        flags = [self._dominance.compare(solution, s) for s in self._contents]
        dominates = [x > 0 for x in flags]
        nondominated = [x == 0 for x in flags]

        if any(dominates):
            return False
        else:
            self._contents = list(itertools.compress(self._contents, nondominated)) + [solution]
            return True

    def append(self, solution):
        self.add(solution)

    def extend(self, solutions):
        for solution in solutions:
            self.append(solution)

    def remove(self, solution):
        try:
            self._contents.remove(solution)
            return True
        except ValueError:
            return False

    def __len__(self):
        return len(self._contents)

    def __getitem__(self, key):
        return self._contents[key]

    def __iadd__(self, other):
        if hasattr(other, "__iter__"):
            for o in other:
                self.add(o)
        else:
            self.add(other)

        return self

    def __iter__(self):
        return iter(self._contents)


class EpsilonBoxArchive(Archive):

    def __init__(self, epsilons):
        super(EpsilonBoxArchive, self).__init__(EpsilonDominance(epsilons))
        self.improvements = 0

    def add(self, solution):
        flags = [self._dominance.compare(solution, s) for s in self._contents]
        dominates = [x > 0 for x in flags]
        nondominated = [x == 0 for x in flags]
        dominated = [x < 0 for x in flags]
        not_same_box = [not self._dominance.same_box(solution, s) for s in self._contents]

        if any(dominates):
            return False
        else:
            self._contents = list(itertools.compress(self._contents, nondominated)) + [solution]

            if dominated and not_same_box:
                self.improvements += 1


class Crossover(Operation):

    def __init__(self, parameters, probability):
        super().__init__()
        self.parameters = parameters
        self.probability = probability

    @abstractmethod
    def cross(self, p1, p2):
        pass


class SimpleCrossover(Crossover):

    def __init__(self, parameters, probability):
        super().__init__(parameters, probability)

    def cross(self, p1, p2):
        parent_a = p1.vector.copy()
        parent_b = p2.vector.copy()

        if random.uniform(0.0, 1.0) <= self.probability:
            parameter1 = []
            parameter2 = []
            linear_range = 2

            alpha = random.uniform(0, linear_range)

            for i, param in enumerate(self.parameters):
                l_b = param['bounds'][0]
                u_b = param['bounds'][1]

                parameter1.append(self.clip(alpha * p1.vector[i] + (1 - alpha) * p2.vector[i], l_b, u_b))
                parameter2.append(self.clip((1 - alpha) * p1.vector[i] + alpha * p2.vector[i], l_b, u_b))

            parent_a = parameter1
            parent_b = parameter2

        offspring_a = p1.__class__(parent_a)
        offspring_b = p2.__class__(parent_b)

        return offspring_a, offspring_b


class SimulatedBinaryCrossover(Crossover):

    def __init__(self, parameters, probability, distribution_index=15):
        super().__init__(parameters, probability)
        self.distribution_index = distribution_index

    def sbx(self, x1, x2, lb, ub, p_type="real"):

        dx = x2 - x1

        if dx > EPSILON:
            if x2 > x1:
                y2 = x2
                y1 = x1
            else:
                y2 = x1
                y1 = x2

            beta = 1.0 / (1.0 + (2.0 * (y1 - lb) / (y2 - y1)))
            alpha = 2.0 - pow(beta, self.distribution_index + 1.0)
            rand = random.uniform(0.0, 1.0)

            if rand <= 1.0 / alpha:
                alpha = alpha * rand
                betaq = pow(alpha, 1.0 / (self.distribution_index + 1.0))
            else:
                alpha = alpha * rand
                alpha = 1.0 / (2.0 - alpha)
                betaq = pow(alpha, 1.0 / (self.distribution_index + 1.0))

            x1 = 0.5 * ((y1 + y2) - betaq * (y2 - y1))
            beta = 1.0 / (1.0 + (2.0 * (ub - y2) / (y2 - y1)))
            alpha = 2.0 - pow(beta, self.distribution_index + 1.0)

            if rand <= 1.0 / alpha:
                alpha = alpha * rand
                betaq = pow(alpha, 1.0 / (self.distribution_index + 1.0))
            else:
                alpha = alpha * rand
                alpha = 1.0 / (2.0 - alpha)
                betaq = pow(alpha, 1.0 / (self.distribution_index + 1.0))

            x2 = 0.5 * ((y1 + y2) + betaq * (y2 - y1))

            # randomly swap the values
            if bool(random.getrandbits(1)):
                x1, x2 = x2, x1

            x1 = self.clip(x1, lb, ub)
            x2 = self.clip(x2, lb, ub)

            if p_type == "integer":
                x1 = int(x1)
                x2 = int(x2)

        return x1, x2

    def cross(self, p1, p2):
        """
        Create an offspring using simulated binary crossover.
        :return:  a list with 2 offsprings each with the genotype of an  offspring after recombination and mutation.
        """

        parent_a = p1.vector.copy()
        parent_b = p2.vector.copy()

        if random.uniform(0.0, 1.0) <= self.probability:
            for i, param in enumerate(self.parameters):
                if random.uniform(0.0, 1.0) <= 0.5:
                    x1 = parent_a[i]
                    x2 = parent_b[i]

                    lb = param['bounds'][0]
                    ub = param['bounds'][1]

                    if not 'parameter_type' in param:
                        continue
                    else:
                        p_type = param['parameter_type']

                    x1, x2 = self.sbx(x1, x2, lb, ub, p_type)

                    parent_a[i] = x1
                    parent_b[i] = x2

        offspring_a = p1.__class__(parent_a)
        offspring_b = p2.__class__(parent_b)

        return offspring_a, offspring_b



from .problem import Problem
from .utils import ConfigDictionary
from .job import Job
from .population import Population
from .individual import Individual,GeneticIndividual
from .operators import RandomGeneration

from abc import ABCMeta, abstractmethod
import numpy as np

from joblib import Parallel, delayed


class Algorithm(metaclass=ABCMeta):
    """ Base class for optimization algorithms. """

    def __init__(self, problem: Problem, name="Algorithm"):
        self.name = name
        self.problem = problem
        self.parameters = problem.parameters
        self.options = ConfigDictionary()

        self.options.declare(name='verbose_level', default=1, lower=0,
                             desc='Verbose level')
        self.options.declare(name='calculate_gradients', default=False,
                             desc='Enable calculating of gradients')
        # max(int(2 / 3 * cpu_count(), 1)
        self.options.declare(name='max_processes', default=1,
                             desc='Max running processes')

        self.options.declare(name='n_iterations', default=10,
                             desc='Max number of iterations')

    def evaluate(self, individuals: list):
        if self.options["max_processes"] > 1:
            self.evaluate_parallel(individuals)
        else:
            self.evaluate_serial(individuals)

        n_failed = 0
        for individual in individuals:
            if individual.state == Individual.State.FAILED:
                n_failed += 1
                individuals.remove(individual)   # TODO: is can be not feasible?
            if isinstance(individual, GeneticIndividual):
                individual.transform_data(self.problem.signs)

    def evaluate_serial(self, individuals: list):
        job = Job(self.problem)
        for individual in individuals:
            if individual.state == Individual.State.EMPTY:
                job.evaluate(individual)

    def evaluate_parallel(self, individuals: list):
        # simple parallel loop
        job = Job(self.problem)
        Parallel(n_jobs=self.options["max_processes"], verbose=1, require='sharedmem')(delayed(job.evaluate)(individual)
                                                                                       for individual in individuals)

    def evaluate_gradient(self, individuals: list, gradient_individuals: list):
        # Todo: Make operator Gradient consisting of current gradient generator and evaluate_gradient
        # individuals.sort(key=lambda x: abs(x.depends_on))
        n_params = len(individuals[0].vector)
        for j in range(len(individuals)):
            table = []
            for i in range(len(gradient_individuals)):
                individual = gradient_individuals[i]
                if individual.depends_on == j:
                    table.append(individual)

            gradient = np.zeros(n_params)
            for k in range(len(table)):
                index = table[k].modified_param
                gradient[index] = ((individuals[j].costs[0] - table[k].costs[0]) / 1e-6)

            individuals[j].gradient = gradient


class DummyAlgorithm(Algorithm):
    """
    Dummy class for testing
    """

    def __init__(self, problem, name='DummyAlgorithm'):
        super().__init__(problem, name)

    def run(self):
        pass




from artap.problem import Problem
from artap.algorithm_genetic import NSGAII, EpsMOEA
from artap.results import Results, GraphicalResults

from math import inf
import matplotlib.pyplot as plt


class TwoBarTrussDesignProblem(Problem):
    """
    Example from K.DEb Multi-objective evolutionary optimization problems, Wiley, 2001.
    pp 432 - 433

    The truss has to carry a certain load without elastic failure. Thus, in addition to the objective of
    designing the truss for minimum volume, there are additional objectives of minimizing the stress in
    each of the two members AC and BC.


    The following two objactive optimization problem should be solved for the three variables:

    x1, x2 are horizontal varibales
    and y is the vertical

    Minimize
    --------

    f1 = x1*sqrt(16+y^2) + x2*sqrt(1+y**2)
    f2 = max(sigmaAC, sigmaBC)

    subject to
    ----------

    max(sigmaAC, sigmaBC) <= 1e5
    1<= x3 <= 3
    x1, x2 >= 0

    """
    def set(self):
        self.parameters = [{'name': 'x_1', 'bounds': [0.0, 0.01]},
                           {'name': 'x_2', 'bounds': [0.0, 0.1]},
                           {'name': 'y', 'bounds': [1.0, 3.0]}]
        self.costs = [{'name': 'F_1'},
                      {'name': 'F_2'}]

    def evaluate(self, individual):
        x = individual.vector

        f1 = x[0]*(16+x[2]**2.)**0.5 + x[1]*(1+x[2]**2.)**0.5

        sigmaAC = 20*(16+x[2]**2.)**0.5/(x[2]*x[0])
        sigmaBC = 80 * (1 + x[2] ** 2.) ** 0.5 / (x[2] * x[1])

        f2 = max(sigmaAC, sigmaBC)

        if f2 > 1e5:
            f2 = inf

        return [f1, f2]


problem = TwoBarTrussDesignProblem()
algorithm = NSGAII(problem)
algorithm.options['max_population_number'] = 100
algorithm.options['max_population_size'] = 100
algorithm.run()

b = Results(problem)
solution = b.pareto_values()

print(solution)

# plot solution
plt.scatter([s[0] for s in solution],
            [s[1]*1e-3 for s in solution])
plt.xlim([0, 0.1])
plt.ylim([0, 100.])
plt.xlabel("Volume [m3]")
plt.ylabel("Maximum stress [MPa]")

# The values from the original solution
# Original solution of the task with the eps-contraint method
# original Palli et al, 1999
plt.scatter(0.004445, 89.983, c='red')
plt.annotate('   Red points are the original solutions from eps-constraint method', (0.004445, 89.983))
#plt.annotate('Original solution with eps-constraint method',(0.004833, 83.268))
plt.scatter(0.004833, 83.268, c='red')

plt.show()


import os

from artap.executor import CondorJobExecutor
from artap.problem import Problem
from artap.algorithm_bayesopt import BayesOptSerial
from artap.algorithm_nlopt import NLopt, LN_BOBYQA

from artap.results import Results, GraphicalResults


class OptProblem(Problem):
    def __init__(self, name):
        # important - nutne prepsat dle modelu (precision neumi algoritmy pouzit klidne smazat)
        """
        parameters = {'A': {'initial_value': 16e-3, 'bounds': [15e-3, 30e-3]},
                      'B': {'initial_value': 20e-3, 'bounds': [16e-3, 30e-3]},
                      'delta': {'initial_value': 2e-3, 'bounds': [1e-3, 5e-3]},
                      'Dind1': {'initial_value': 180e-3, 'bounds': [170e-3, 300e-3]},
                      'C_par': {'initial_value': 3e-3, 'bounds': [2e-3, 10e-3]},
                      'D_par': {'initial_value': 3e-3, 'bounds': [2e-3, 10e-3]}}
        """
        parameters = {'A': {'initial_value': 16e-3, 'bounds': [16e-3, 16.1e-3]},
                      'B': {'initial_value': 20e-3, 'bounds': [20e-3, 20e-3]},
                      'delta': {'initial_value': 2e-3, 'bounds': [1.9e-3, 2.1e-3]},
                      'Dind1': {'initial_value': 180e-3, 'bounds': [180e-3, 180e-3]},
                      'C_par': {'initial_value': 3e-3, 'bounds': [3e-3, 3e-3]},
                      'D_par': {'initial_value': 3e-3, 'bounds': [3e-3, 3e-3]}}

        # cilove funkce a jejich jmena
        costs = ['F']
        # kde je model?
        working_dir = os.getcwd() + os.sep + "hes_ivan"
        # os.remove(working_dir + "data.sqlite")

        super().__init__(name, parameters, costs, working_dir=working_dir)

        # nemenit - pousti se comsol na condoru
        self.executor = CondorJobExecutor(self,
                                          command="run.sh",
                                          model_file="HES_titan_opt2_artap.mph",
                                          output_file="outfile.txt",
                                          supplementary_files=["run.sh"])

    def evaluate(self, x):
        result = self.executor.eval(x)

        # DEFINICE CILOVE FUNKCE
        of = abs(result[1] - result[0])
        # VYSTUP NA OBR - LZE MENIT
        print(x, result, of)

        return [of]

    def parse_results(self, content):
        lines = content.split("\n")
        line_with_results1 = lines[6].split()  # 5th line contains result
        line_with_results2 = lines[7].split()  # 5th line contains result
        print(line_with_results1)
        min = float(line_with_results1[2])
        max = float(line_with_results2[2])

        return [min, max]


if __name__ == '__main__':
    problem = OptProblem("Opt")

    algorithm = NLopt(problem)
    algorithm.options['algorithm'] = LN_BOBYQA

    # algorithm = BayesOptSerial(problem)
    algorithm.options['verbose_level'] = 1
    algorithm.options['n_iterations'] = 2
    algorithm.run()

    results = Results(problem)

    results = GraphicalResults(problem)
    optimum = results.find_minimum(name='F')
    results.plot_scatter('F_1', 'F_2', filename="/tmp/scatter.pdf")
    # results.plot_scatter('x_1', 'x_2')
    # results.plot_individuals('F_1')



import unittest
from unittest import TestCase, main

from artap.executor import CondorComsolJobExecutor, CondorMatlabJobExecutor, CondorPythonJobExecutor
from artap.problem import Problem
from artap.population import Population
from artap.algorithm import DummyAlgorithm
from artap.individual import Individual
from artap.config import config

import random


class CondorMatlabProblem(Problem):
    """ Describe simple one objective optimization problem. """

    def set(self):
        self.name = "CondorMatlabProblem"
        self.parameters = [{'name': 'a', 'initial_value': 10, 'bounds': [0, 20]},
                      {'name': 'b', 'initial_value': 10, 'bounds': [5, 15]}]
        self.costs = [{'name': 'F_1'}]
        self.executor = CondorMatlabJobExecutor(self,
                                                script="./data/run_input.m",
                                                parameter_file="input.txt",
                                                files_from_condor=["output.txt"])

    def evaluate(self, individual):
        return self.executor.eval(individual)

    def parse_results(self, output_files, individual):
        with open(output_files[0]) as file:
            content = file.readlines()
        return [float(content[0])]


class CondorComsolProblem(Problem):
    """ Describe simple one objective optimization problem. """

    def set(self):
        self.name = "CondorComsolProblem"
        self.parameters = [{'name': 'a', 'initial_value': 10, 'bounds': [0, 20]},
                           {'name': 'b', 'initial_value': 10, 'bounds': [5, 15]}]

        self.costs = [{'name': 'F_1'}]
        self.executor = CondorComsolJobExecutor(self, model_file="./data/elstat.mph",
                                                files_from_condor=["out.txt", "elstat.mph"])

    def evaluate(self, individual):
        return self.executor.eval(individual)

    def parse_results(self, output_files, individual):
        with open(output_files[0]) as file:
            content = file.readlines()
        line_with_results = content[5]  # 5th line contains results
        result = float(line_with_results)
        return [result]


class PythonExecProblem(Problem):
    """ Describe simple one objective optimization problem. """
    def set(self):
        self.parameters = [{'name': 'a', 'initial_value': 10, 'bounds': [0, 20]},
                           {'name': 'b', 'initial_value': 10, 'bounds': [5, 15]}]
        self.costs = [{'name': 'F_1'}]
        self.executor = CondorPythonJobExecutor(self,
                                                script="./data/run_exec.py",
                                                parameter_file=None,
                                                output_files=["output.txt"])

    def evaluate(self, individual):
        return self.executor.eval(individual)

    def parse_results(self, output_files, individual):
        with open(output_files[0]) as file:
            content = file.readlines()
        return [float(content[0])]


class PythonInputProblem(Problem):
    """ Describe simple one objective optimization problem. """
    def set(self):
        self.parameters = [{'name': 'a', 'initial_value': 10, 'bounds': [0, 20]},
                           {'name': 'b', 'initial_value': 10, 'bounds': [5, 15]}]
        self.costs = [{'name': 'F_1'}]
        self.executor = CondorPythonJobExecutor(self,
                                                script="./data/run_input.py",
                                                parameter_file="input.txt",
                                                output_files=["output.txt"])

    def evaluate(self, individual):
        return self.executor.eval(individual)

    def parse_results(self, output_files, individual):
        with open(output_files[0]) as file:
            content = file.readlines()
        return [float(content[0])]


class TestCondor(TestCase):
    """ Tests simple optimization problem where calculation of
        goal function is submitted as a job on HtCondor.
    """

    @unittest.skipIf(config["condor_host"] is None, "Condor is not defined.")
    def test_condor_matlab_input(self):
        """ Tests one calculation of goal function."""
        problem = CondorMatlabProblem()

        individuals = [Individual([1, 2])]
        evaluator = DummyAlgorithm(problem)
        evaluator.evaluate(individuals)

        self.assertAlmostEqual(5, individuals[0].costs[0])

    @unittest.skipIf(config["condor_host"] is None, "Condor is not defined.")
    def test_condor_comsol_exec(self):
        """ Tests one calculation of goal function."""
        problem = CondorComsolProblem()

        individuals = [Individual([10, 10]), Individual([11, 11])]
        evaluator = DummyAlgorithm(problem)
        evaluator.evaluate(individuals)

        self.assertAlmostEqual(112.94090668383139, individuals[0].costs[0])
        self.assertAlmostEqual(124.23499735221547, individuals[1].costs[0])

    @unittest.skipIf(config["condor_host"] is None, "Condor is not defined.")
    def test_condor_python_exec(self):
        problem = PythonExecProblem()

        individuals = [Individual([1, 2])]
        evaluator = DummyAlgorithm(problem)
        evaluator.evaluate(individuals)

        self.assertAlmostEqual(5, individuals[0].costs[0])

    @unittest.skipIf(config["condor_host"] is None, "Condor is not defined.")
    def test_condor_python_exec_full_load(self):
        problem = PythonExecProblem()

        n = 200
        individuals = []
        for i in range(n):
            individuals.append(Individual([random.randrange(1, 100), random.randrange(1, 100)]))

        algorithm = DummyAlgorithm(problem)
        algorithm.options['max_processes'] = 70
        algorithm.evaluate(individuals)

        for i in range(n):
            # print(individuals[i])
            self.assertEqual(int(individuals[i].costs[0]), individuals[i].vector[0]**2 + individuals[i].vector[1]**2)

    @unittest.skipIf(config["condor_host"] is None, "Condor is not defined.")
    def test_condor_python_input(self):
        problem = PythonInputProblem()

        individuals = [Individual([1, 2])]
        evaluator = DummyAlgorithm(problem)
        evaluator.evaluate(individuals)

        self.assertAlmostEqual(5, individuals[0].costs[0])


if __name__ == '__main__':
    main()



import math
import unittest

from artap.problem import Problem
from artap.individual import Individual
from artap.benchmark_functions import Booth
from artap.surrogate import SurrogateModelEval
from artap.surrogate_scikit import SurrogateModelScikit
from artap.surrogate_smt import SurrogateModelSMT
from artap.operators import LHSGeneration
from artap.algorithm_sweep import SweepAlgorithm

from sklearn.gaussian_process import GaussianProcessRegressor
from sklearn.gaussian_process.kernels import RationalQuadratic, ExpSineSquared
# from sklearn.gaussian_process.kernels RBF, Matern, DotProduct, WhiteKernel, ConstantKernel
from sklearn.ensemble import BaggingRegressor, RandomForestRegressor, GradientBoostingRegressor


class MyProblemSin(Problem):
    """ Describe simple one objective optimization problem. """

    def set(self):
        self.parameters = [{'name': 'x_1', 'initial_value': 2.5, 'bounds': [0, 10]}]
        self.costs = [{'name': 'F'}]

    def evaluate(self, individual):
        x = individual.vector
        return [x[0] * math.sin(x[0])]


# class MyProblemBooth(Problem):
#     """ Describe simple one objective optimization problem. """
#
#     def set(self):
#         self.parameters = [{'name': 'x_1', 'initial_value': 0, 'bounds': [-5, 5]},
#                            {'name': 'x_2', 'initial_value': 0, 'bounds': [-5, 5]}]
#         self.costs = [{'name': 'F'}]
#
#     def evaluate(self, individual):
#         return [Booth.eval(individual.vector)]


class TestSurrogate(unittest.TestCase):
    def _check_scikit_one(self, problem, threshold=5.0):
        xmeas = [0., 0.5, 1., 2., 3., 4., 5., 5.4, 6., 7., 7.7, 8., 10.]

        problem.surrogate.train_step = len(xmeas)

        # train
        for val in xmeas:
            problem.surrogate.evaluate(Individual([val]))

        x_ref = Individual([5.1])
        # eval reference
        value_problem = problem.evaluate(x_ref)[0]

        # more steps - some methods are stochastic
        step = 1
        while True:
            # eval surrogate
            value_surrogate = problem.surrogate.predict(x_ref.vector)[0]

            percent = 100.0 * math.fabs(value_problem - value_surrogate) / math.fabs(value_problem)
            problem.logger.info("{}: surrogate.value: step {})".format(problem.name, step))
            problem.logger.info(
                "{}: eval = {}, pred = {}, diff = {} ({} %)".format(problem.name, value_problem, value_surrogate,
                                                                    math.fabs(value_problem - value_surrogate),
                                                                    percent))
            problem.logger.info("{}: score = {}, lml = {}, lml_grad = {}".format(problem.name, problem.surrogate.score,
                                                                                 problem.surrogate.lml,
                                                                                 problem.surrogate.lml_gradient))

            if percent < threshold:
                self.assertLess(percent, threshold)
                break
            else:
                problem.surrogate.train()
                step += 1

    def test_eval(self):
        problem = Booth()#MyProblemBooth()
        problem.surrogate = SurrogateModelEval(problem)

        x_ref = Individual([2.5, 1.5])
        # eval reference
        value_problem = problem.evaluate(x_ref)[0]
        # eval surrogate
        value_surrogate = problem.surrogate.predict(x_ref)[0]

        problem.logger.info(
            "{}: surrogate.value: evaluation = {}, prediction = {}, difference = {}".format(problem.name,
                                                                                            value_problem,
                                                                                            value_surrogate,
                                                                                            math.fabs(value_problem -
                                                                                                      value_surrogate)))
        self.assertLess(math.fabs(value_problem - value_surrogate), 1e-8)

    def test_scikit_gaussian_process_one(self):
        problem = MyProblemSin()
        problem.surrogate = SurrogateModelScikit(problem)
        problem.surrogate.sigma_threshold = 0.1

        kernel = 1.0 * ExpSineSquared(length_scale=1.0, periodicity=3.0, length_scale_bounds=(0.1, 10.0),
                                      periodicity_bounds=(1.0, 10.0))
        problem.surrogate.regressor = GaussianProcessRegressor(kernel=kernel)

        self._check_scikit_one(problem, 5.0)

    def test_scikit_gradient_boosting_regressor_one(self):
        problem = MyProblemSin()
        problem.surrogate = SurrogateModelScikit(problem)

        problem.surrogate.regressor = GradientBoostingRegressor()

        self._check_scikit_one(problem, 7.0)

    def test_scikit_bagging_regressor_one(self):
        problem = MyProblemSin()
        problem.surrogate = SurrogateModelScikit(problem)

        problem.surrogate.regressor = BaggingRegressor()

        self._check_scikit_one(problem, 7.0)

    def test_scikit_random_tree_regressor_one(self):
        problem = MyProblemSin()
        problem.surrogate = SurrogateModelScikit(problem)

        problem.surrogate.regressor = RandomForestRegressor(n_estimators=20)

        self._check_scikit_one(problem, 10.0)

    def test_scikit_gaussian_process_two(self):
        problem = Booth()
        problem.surrogate = SurrogateModelScikit(problem)
        # set custom regressor
        kernel = 1.0 * RationalQuadratic(length_scale=1.0, alpha=0.1)
        problem.surrogate.regressor = GaussianProcessRegressor(kernel=kernel)
        # set threshold
        problem.surrogate.sigma_threshold = 0.01
        problem.surrogate.train_step = 12

        # train
        problem.surrogate.evaluate(Individual([1.0, 3.1]))
        problem.surrogate.evaluate(Individual([1.3, 3.2]))
        problem.surrogate.evaluate(Individual([1.8, 2.7]))
        problem.surrogate.evaluate(Individual([0.9, 3.3]))
        problem.surrogate.evaluate(Individual([0.8, 3.1]))
        problem.surrogate.evaluate(Individual([1.4, 2.8]))
        problem.surrogate.evaluate(Individual([0.1, 3.0]))
        problem.surrogate.evaluate(Individual([0.2, 3.4]))
        problem.surrogate.evaluate(Individual([0.95, 3.03]))
        problem.surrogate.evaluate(Individual([0.98, 2.96]))
        problem.surrogate.evaluate(Individual([1.02, 2.98]))
        problem.surrogate.evaluate(Individual([1.01, 2.99]))

        x_ref = Individual([0.2, 3.1])
        # eval reference
        value_problem = problem.evaluate(x_ref)[0]
        # eval surrogate
        value_surrogate = problem.surrogate.predict(x_ref.vector)[0]

        percent = 100.0 * math.fabs(value_problem - value_surrogate) / math.fabs(value_problem)
        problem.logger.info(
            "{}: surrogate.value: eval = {}, pred = {}, diff = {} ({} %)".format(problem.name, value_problem,
                                                                                 value_surrogate,
                                                                                 math.fabs(
                                                                                     value_problem - value_surrogate),
                                                                                 percent))

        self.assertLess(percent, 5.0)

    def test_scikit_gaussian_process_lhs_two(self):
        problem = Booth()
        problem.set_init_values(**{'initial_value':[0.,0.]})
        problem.surrogate = SurrogateModelScikit(problem)
        # set custom regressor
        kernel = 1.0 * RationalQuadratic(length_scale=1.0)
        problem.surrogate.regressor = GaussianProcessRegressor(kernel=kernel)

        # set threshold
        problem.surrogate.sigma_threshold = 0.01
        problem.surrogate.train_step = 100

        # sweep analysis (for training)
        gen = LHSGeneration(problem.parameters)
        gen.init(problem.surrogate.train_step)
        algorithm_sweep = SweepAlgorithm(problem, generator=gen)
        algorithm_sweep.run()

        x_ref = Individual([2.00, -2.00])
        # eval reference
        value_problem = problem.evaluate(x_ref)[0]
        # eval surrogate
        value_surrogate = problem.surrogate.predict(x_ref.vector)[0]

        percent = 100.0 * math.fabs(value_problem - value_surrogate) / math.fabs(value_problem)
        problem.logger.info(
            "{}: surrogate.value: eval = {}, pred = {}, diff = {} ({} %)".format(problem.name, value_problem,
                                                                                 value_surrogate,
                                                                                 math.fabs(
                                                                                     value_problem - value_surrogate),
                                                                                 percent))

        self.assertLess(percent, 5.0)

    def _check_smt(self, regressor, percent=1.0):
        problem = Booth()
        problem.surrogate = SurrogateModelSMT(problem)
        # set custom regressor
        problem.surrogate.regressor = regressor
        # set threshold
        problem.surrogate.train_step = 12

        # train
        problem.surrogate.evaluate(Individual([1.0, 3.1]))
        problem.surrogate.evaluate(Individual([1.3, 3.2]))
        problem.surrogate.evaluate(Individual([1.8, 2.7]))
        problem.surrogate.evaluate(Individual([0.9, 3.3]))
        problem.surrogate.evaluate(Individual([0.8, 3.1]))
        problem.surrogate.evaluate(Individual([1.4, 2.8]))
        problem.surrogate.evaluate(Individual([0.1, 3.0]))
        problem.surrogate.evaluate(Individual([0.2, 3.4]))
        problem.surrogate.evaluate(Individual([0.95, 3.03]))
        problem.surrogate.evaluate(Individual([0.98, 2.96]))
        problem.surrogate.evaluate(Individual([1.02, 2.98]))
        problem.surrogate.evaluate(Individual([1.01, 2.99]))

        # check values
        x_ref = Individual([0.2, 3.1])
        # eval reference
        value_problem = problem.evaluate(x_ref)[0]
        # eval surrogate
        value_surrogate = problem.surrogate.predict(x_ref.vector)

        percent_check = 100.0 * math.fabs(value_problem - value_surrogate) / math.fabs(value_problem)
        problem.logger.info(
            "{}: surrogate.value: eval = {}, pred = {}, diff = {} ({} %)".format(problem.name, value_problem,
                                                                                 value_surrogate,
                                                                                 math.fabs(
                                                                                     value_problem - value_surrogate),
                                                                                 percent_check))

        self.assertLess(percent_check, percent)

    def test_smt_rbf(self):
        self._check_smt(SurrogateModelSMT.get_rbf_regressor())

    #def test_smt_idw(self):
    #    self._check_smt(SurrogateModelSMT.get_idw_regressor())

    def test_smt_kpls(self):
        self._check_smt(SurrogateModelSMT.get_kpls_regressor())

    def test_smt_kplsk(self):
        self._check_smt(SurrogateModelSMT.get_kplsk_regressor())

    #def test_smt_ls(self):
    #    self._check_smt(SurrogateModelSMT.get_ls_regressor())

    def test_smt_qp(self):
        self._check_smt(SurrogateModelSMT.get_qp_regressor())

    def test_smt_krg(self):
        self._check_smt(SurrogateModelSMT.get_krg_regressor())


if __name__ == '__main__':
    unittest.main()



import unittest

try:
    from artap.algorithm_nlopt import NLopt
    from artap.algorithm_nlopt import GN_DIRECT_L
    from artap.algorithm_nlopt import GN_MLSL
    from artap.algorithm_nlopt import GN_CRS2_LM
    from artap.algorithm_nlopt import GN_ISRES
    from artap.algorithm_nlopt import LN_BOBYQA
    from artap.algorithm_nlopt import LN_COBYLA
    from artap.algorithm_nlopt import LN_NELDERMEAD
    from artap.algorithm_nlopt import LN_SBPLX
    from artap.algorithm_nlopt import LN_PRAXIS
    from artap.algorithm_nlopt import LN_AUGLAG_EQ

    __nlopt__ = True
except ImportError:
    __nlopt__ = False

from artap.results import Results
from artap.benchmark_functions import Booth


class TestNLoptOptimization(unittest.TestCase):
    """ Tests simple one objective optimization problem."""

    def run_test(self, method, n_iterations=100):
        try:
            problem = Booth()
            algorithm = NLopt(problem)
            algorithm.options['verbose_level'] = 0
            algorithm.options['algorithm'] = method
            algorithm.options['xtol_abs'] = 1e-6
            algorithm.options['xtol_rel'] = 1e-3
            algorithm.options['ftol_rel'] = 1e-3
            algorithm.options['ftol_abs'] = 1e-6
            algorithm.options['n_iterations'] = n_iterations
            algorithm.run()

            results = Results(problem)
            optimum = results.find_minimum('F')
            self.assertAlmostEqual(optimum.costs[0], 0, places=1)
        except AssertionError:
            # try again with more iterations - NLopt - 3rd party library
            self.run_test(method, int(1.5 * n_iterations))

    @unittest.skipIf(__nlopt__ is False, "requires NLopt")
    def test_local_problem_nlopt_GN_DIRECT_L(self):
        self.run_test(GN_DIRECT_L, 400)

    # @unittest.skipUnless(sys.platform.startswith("linux"), "requires linux")
    # def test_local_problem_nlopt_GN_DIRECT_L_RAND(self):
    #    self.run_test(GN_DIRECT_L_RAND, 100)

    @unittest.skipIf(__nlopt__ is False, "requires NLopt")
    def test_local_problem_nlopt_GN_MLSL(self):
        self.run_test(GN_MLSL, 500)

    @unittest.skipIf(__nlopt__ is False, "requires NLopt")
    def test_local_problem_nlopt_GN_CRS2_LM(self):
        self.run_test(GN_CRS2_LM, 1000)

    @unittest.skipIf(__nlopt__ is False, "requires NLopt")
    def test_local_problem_nlopt_GN_ISRES(self):
        self.run_test(GN_ISRES, 1000)

    # @unittest.skipUnless(sys.platform.startswith("linux"), "requires linux")
    # def test_local_problem_nlopt_GN_ESCH(self):
    #     self.run_test(GN_ESCH, 5000)

    @unittest.skipIf(__nlopt__ is False, "requires NLopt")
    def test_local_problem_nlopt_LN_BOBYQA(self):
        self.run_test(LN_BOBYQA)

    @unittest.skipIf(__nlopt__ is False, "requires NLopt")
    def test_local_problem_nlopt_LN_COBYLA(self):
        self.run_test(LN_COBYLA)

    @unittest.skipIf(__nlopt__ is False, "requires NLopt")
    def test_local_problem_nlopt_LN_NELDERMEAD(self):
        self.run_test(LN_NELDERMEAD)

    @unittest.skipIf(__nlopt__ is False, "requires NLopt")
    def test_local_problem_nlopt_LN_SBPLX(self):
        self.run_test(LN_SBPLX, 200)

    @unittest.skipIf(__nlopt__ is False, "requires NLopt")
    def test_local_problem_nlopt_LN_AUGLAG_EQ(self):
        self.run_test(LN_AUGLAG_EQ)

    @unittest.skipIf(__nlopt__ is False, "requires NLopt")
    def test_local_problem_nlopt_LN_PRAXIS(self):
        self.run_test(LN_PRAXIS)


if __name__ == '__main__':
    unittest.main()



import unittest
from artap.problem import Problem
from artap.algorithm_firefly import MoFirefly
from artap.results import Results

import optproblems as optp
import optproblems.cec2005 as cec2005


class CEC2005_TEST_Problems(Problem):
    """
    Template class to initialize the different test problems these problems working for 2, 10, 30 variables.

    There are 25 test problems, which were compiled for the Special Session on Real-Parameter Optimization at the
    Congress on Evolutionary Computation (CEC), Edinburgh, UK, 2-5 Sept. 2005. The mathematical definitions are given in
    [Suganthan2005]. Additionally, implementations in C, Java, and Matlab were provided for the participants.
    Artap uses the optproblems test library to run these test. This template class is made to invoke them.
    """

    def set(self):
        # two variables
        self.parameters = [{'name': 'x_1', 'bounds': [func.min_bounds[0], func.max_bounds[0]]},
                           {'name': 'x_2', 'bounds': [func.min_bounds[1], func.max_bounds[1]]}]

        self.costs = [{'name':'F_1'}]


    def evaluate(self, x):
        problem = optp.Problem(func, num_objectives=1)
        solutions = [optp.Individual(x.vector)]

        problem.batch_evaluate(solutions)

        return [solutions[0].objective_values]


class TestCEC2005(unittest.TestCase):
    """Tests from CEC2005 library"""

    def run_test_problem(self, nr_gen, function):
        try:
            global func
            func = function
            problem = CEC2005_TEST_Problems()
            algorithm = MoFirefly(problem)
            algorithm.options['max_population_number'] = nr_gen
            algorithm.options['max_population_size'] = 100
            algorithm.run()

            b = Results(problem)
            optimum = b.find_minimum('F_1')  # Takes last cost function
            self.assertAlmostEqual(optimum.costs[0], func.bias, 0)
            del func
        except AssertionError:
            # try again with more populations
            max_pop = int(1.5 * nr_gen)
            # print("Try again with more populations: {}".format(max_pop))
            self.run_test_problem(max_pop, function)

    def xtest_shifted_sphere(self):
        self.run_test_problem(50, optp.cec2005.F1(2))

    def xtest_shifted_rot_weierstrass(self):
        self.run_test_problem(50, optp.cec2005.F11(2))

    def xtest_fletcherpowell(self):  # schwefel
        self.run_test_problem(50, optp.cec2005.F12(2))

    def xtest_f8f2(self):
        self.run_test_problem(50, optp.cec2005.F13(2))

    def xtest_expanded_f6(self):
        self.run_test_problem(50, optp.cec2005.F14(2))


if __name__ == '__main__':
    unittest.main()



import unittest

from artap.algorithm_scipy import ScipyOpt
from artap.benchmark_functions import Ackley,GramacyLee,AlpineFunction
from artap.results import Results


class TestSimpleOptimization(unittest.TestCase):
    """ Tests simple one objective optimization problem."""

    def test_local_problem(self):
        problem = AlpineFunction(**{'dimension':1, 'initial_value': 1.})
        algorithm = ScipyOpt(problem)
        algorithm.options['algorithm'] = 'Nelder-Mead'
        algorithm.options['tol'] = 1e-4
        algorithm.run()

        results = Results(problem)
        optimum = results.find_minimum('F_1')
        self.assertAlmostEqual(optimum.costs[0], 0)


class TestAckleyN2(unittest.TestCase):
    """ Tests simple one objective optimization problem."""

    def test_local_problem(self):
        problem = Ackley(**{'dimension':1})
        algorithm = ScipyOpt(problem)
        algorithm.options['algorithm'] = 'Nelder-Mead'
        algorithm.options['tol'] = 1e-4
        algorithm.options['calculate_gradients'] = True
        algorithm.run()

        print(problem.populations[0].individuals[3])

        results = Results(problem)
        optimum = results.find_minimum('F_1')
        self.assertAlmostEqual(optimum.costs[0], problem.global_optimum, 3)


if __name__ == '__main__':
    unittest.main()



import os
import getpass
from appdirs import user_config_dir
import pathlib

# internal variables
__id__ = "artap"
path = str(pathlib.Path(__file__).parent.absolute())

artap_root = path
tests_root = os.path.abspath(path + os.sep + "tests" + os.sep)

# config
config = {}

# config["condor_host"] = "edison.fel.zcu.cz"
config["condor_host"] = None
# get system user name
config["condor_login"] = getpass.getuser()

config["comsol_path"] = ""

config["loopback_ip"] = "127.0.0.1"
config["server_initial_port"] = 8050
config["server_keep_live_delay"] = 0.1

# user config
fn = "{}/config.py".format(user_config_dir(__id__))
if os.path.exists(fn):
    exec(open(fn).read())


import unittest

try:
    from artap.algorithm_bayesopt import BayesOptSerial, BayesOptParallel

    __bayes_opt__ = True
except ImportError:
    __bayes_opt__ = False

from artap.results import Results
from artap.benchmark_functions import Booth


class TestBayesOptOptimization(unittest.TestCase):
    """ Tests simple one objective optimization problem."""

    # def test_local_problem_bayesopt_parallel(self):
    #     problem = Booth()
    #     algorithm = BayesOptParallel(problem)
    #     algorithm.options['verbose_level'] = 0
    #     algorithm.options['n_iterations'] = 50
    #     algorithm.run()
    #     # TODO - multiprocess test
    #
    #     results = Results(problem)
    #     optimum = results.find_minimum(name='F')
    #     self.assertAlmostEqual(optimum.costs[0], 0, places=2)

    @unittest.skipIf(__bayes_opt__ is False, "requires module BayesOpt")
    def test_local_problem_bayesopt_serial(self):
        problem = Booth()
        algorithm = BayesOptSerial(problem)
        algorithm.options['verbose_level'] = 0
        algorithm.options['n_iterations'] = 200
        algorithm.run()

        results = Results(problem)
        optimum = results.find_minimum(name='F')
        self.assertAlmostEqual(optimum.costs[0], 0, places=2)


if __name__ == '__main__':
    unittest.main()



from .utils import VectorAndNumbers
from .algorithm import Algorithm
from .population import Population
from .individual import Individual
from .job import Job
from .problem import Problem
from .operators import CustomGeneration

from SALib.sample.saltelli import sample as sobol_sample
from SALib.sample.morris import sample as morris_sample
from SALib.sample.fast_sampler import sample as fast_sample
from SALib.sample.latin import sample as latin_sample
from SALib.sample.ff import sample as ff_sample

from SALib.analyze import sobol
from SALib.analyze import ff
from SALib.analyze import morris
from SALib.analyze import fast
from SALib.analyze import rbd_fast
from SALib.analyze import delta

import time
import numpy as np

_method = ['rbd_fast', 'fast', 'morris', 'sobol', 'delta', 'ff']


class SALibAlgorithm(Algorithm):
    """
    SALib Analysis
    """

    def __init__(self, problem: Problem, name='SALibAlgorithm'):
        super().__init__(problem, name)

        self.sa_problem = {}
        self.samples_x = []
        self.samples_y = []

        self.job = Job(self.problem)

        self.options.declare(name='method', default='sobol', values=_method,
                             desc='Method')
        self.options.declare(name='print_to_console', default=False,
                             desc='Print to console')
        self.options.declare(name='samples', default=10, lower=1,
                             desc='Samples')

    def run(self):
        t_s = time.time()

        # set SALib problem
        names = []
        bounds = []
        for parameter in self.problem.parameters:
            names.append(parameter['name'])
            bounds.append(parameter['bounds'])

        self.sa_problem = {'num_vars': len(self.problem.parameters),
                           'names': names,
                           'bounds': bounds}

        # generate samples
        if self.options["method"] == "rbd_fast":
            self.samples_x = latin_sample(self.sa_problem, self.options["samples"])
        elif self.options["method"] == "fast":
            self.samples_x = fast_sample(self.sa_problem, self.options["samples"])
        elif self.options["method"] == "morris":
            self.samples_x = morris_sample(self.sa_problem, self.options["samples"], num_levels=4)
        elif self.options["method"] == "sobol":
            self.samples_x = sobol_sample(self.sa_problem, self.options["samples"])
        elif self.options["method"] == "delta":
            self.samples_x = latin_sample(self.sa_problem, self.options["samples"])
        elif self.options["method"] == "ff":
            self.samples_x = ff_sample(self.sa_problem, self.options["samples"])

        individuals = []
        for vector in self.samples_x:
            individuals.append(Individual(vector))

        # set current size
        self.population_size = len(individuals)
        # evaluate individuals
        self.evaluate(individuals)

        for individual in individuals:
            self.samples_y.append(individual.costs[0]) # TODO: fix index [0]
        self.samples_y = np.array(self.samples_y)

        population = Population(individuals)
        self.problem.populations.append(population)

        t = time.time() - t_s
        self.problem.logger.info("Sweep: elapsed time: {} s".format(t))

    def analyze(self):
        if self.options["method"] == "rbd_fast":
            return self.analyze_rbd_fast()
        elif self.options["method"] == "fast":
            return self.analyze_sobol()
        elif self.options["method"] == "morris":
            return self.analyze_morris()
        elif self.options["method"] == "sobol":
            return self.analyze_sobol()
        elif self.options["method"] == "delta":
            return self.analyze_delta()
        elif self.options["method"] == "ff":
            return self.analyze_ff()

    def analyze_rbd_fast(self):
        #  RBD-FAST - Random Balance Designs Fourier Amplitude Sensitivity Test
        return rbd_fast.analyze(self.sa_problem, self.samples_x, self.samples_y, print_to_console=self.options["print_to_console"])

    def analyze_fast(self):
        # FAST - Fourier Amplitude Sensitivity Test
        return fast.analyze(self.sa_problem, self.samples_y, print_to_console=self.options["print_to_console"])

    def analyze_morris(self):
        # Method of Morris
        return morris.analyze(self.sa_problem, self.samples_x, self.samples_y, conf_level=0.95, num_levels=4, print_to_console=self.options["print_to_console"])

    def analyze_sobol(self):
        # Sobol Sensitivity Analysis
        return sobol.analyze(self.sa_problem, self.samples_y, print_to_console=self.options["print_to_console"])

    def analyze_delta(self):
        # Delta Moment-Independent Measure
        return delta.analyze(self.sa_problem, self.samples_x, self.samples_y, print_to_console=self.options["print_to_console"])

    def analyze_ff(self):
        # Fractional Factorial
        return ff.analyze(self.sa_problem, self.samples_x, self.samples_y, second_order=True, print_to_console=self.options["print_to_console"])


class Sensitivity(Algorithm):
    def __init__(self, problem, parameters, name='Sensitivity analysis'):
        self.parameters = parameters
        super().__init__(problem, name)

        self.options.declare(name='max_population_size', default=100, lower=1,
                             desc='Maximal number of individuals in population')

    def run(self):
        self.population_size = self.options['max_population_size']

        parameters = []
        for parameter in self.problem.parameters:
            parameters.append(float(parameter['initial_value']))

        population = Population()

        for parameter_name in self.parameters:
            parameter_values = []

            index = 0
            selected_parameter = None
            for parameter in self.parameters:
                if parameter['name'] == parameter_name['name']:
                    selected_parameter = parameter
                    break
                index += 1

            individuals = []
            for i in range(self.options['max_population_size']):
                value = VectorAndNumbers.gen_number(selected_parameter['bounds'], selected_parameter['precision'],
                                              'normal')
                parameters[index] = value
                parameter_values.append(value)
                individual = Individual(parameters.copy())
                individuals.append(individual)

            self.evaluate(population.individuals)
            costs = []
            # TODO: Make also for multi-objective
            for individual in population.individuals:
                costs.append(individual.costs)

        # append population
        self.problem.populations.append(population)



import os
import time
import rpyc
import datetime
import getopt
import sys

MONITOR_PORT = 16000
port = MONITOR_PORT
MONITOR_SERVER = "localhost"
server = MONITOR_SERVER

options, remainder = getopt.getopt(sys.argv[1:], "h:s:p:", ["help", "server=", "port="])

for opt, args in options:
    if opt in ('-s', '--server'):
        # print("server", args)
        server = args
    elif opt in ('-p', '--port'):
        print("port", args)
        port = int(args)
    elif opt in ('-h', '--help'):
        print("Usage: artap_monitor\n")
        print("\t-s, --server\t\tserver")
        print("\t-p, --port\t\tport")
        exit(0)


def clear():
    os.system('clear' if os.name == 'nt' else 'clear')


# connect to server
conn = rpyc.classic.connect(server, port)
while True:
    name = conn.root.problem.name
    parameters = conn.root.problem.parameters
    costs = conn.root.problem.costs
    stats = conn.root.stats()
    populations = conn.root.populations()

    # clear
    clear()

    print("Artap Monitor - {}".format(name))
    print("")

    # stats
    print("Individuals: {}\t\tNumber of populations: {}".format(stats["individuals_count"], stats["populations_count"]))
    print("Empty:       {}".format(stats["individuals_stats"]["empty"]))
    print("In progress: {}".format(stats["individuals_stats"]["in_progress"]))
    print("Evaluated:   {}".format(stats["individuals_stats"]["evaluated"]))
    print("Failed:      {}".format(stats["individuals_stats"]["failed"]))
    print("")
    parameters_str = ""
    for parameter in parameters:
        parameters_str += "{}: {}, ".format(parameter["name"], parameter["bounds"])
    if len(parameters_str):
        parameters_str = parameters_str[:-2]
    costs_str = ""
    for cost in costs:
        costs_str += "{}, ".format(cost["name"])
        # costs_str += "{}: {}, ".format(cost["name"], cost["criteria"])
    if len(costs_str):
        costs_str = costs_str[:-2]
    print("Parameters: {}".format(parameters_str))
    print("Costs:      {}".format(costs_str))
    print("")

    # individuals
    if len(populations) > 0:
        last_population = populations[-1]
        if last_population is not None:
            template = "{:13s}{:11}{:11}{:13}{:" + str(len(parameters) * 10 + 3) + "s}{:" + str(len(costs) * 10 + 3) + "s}"

            print(template.format("State", "Start", "Finish", "Elapsed", "Parameters", "Costs"))
            for individual in last_population.individuals:
                state = individual.state
                state_str = "empty"
                if state.value == 1:
                    state_str = "in progress"
                elif state.value == 2:
                    state_str = "evaluated"
                elif state.value == 3:
                    state_str = "failed"

                if individual.info["start_time"] > 0:
                    dt_startime = datetime.datetime.fromtimestamp(individual.info["start_time"]).strftime("%H.%M.%S")
                else:
                    dt_startime = ""
                if individual.info["finish_time"] > 0:
                    dt_finishtime = datetime.datetime.fromtimestamp(individual.info["finish_time"]).strftime("%H.%M.%S")
                    dt_elapsedtime = (datetime.datetime.fromtimestamp(individual.info["finish_time"]) - datetime.datetime.fromtimestamp(individual.info["start_time"])).microseconds / 1000
                else:
                    dt_finishtime = ""
                    dt_elapsedtime = datetime.timedelta().microseconds * 1000

                hours, rem = divmod(dt_elapsedtime, 3600)
                minutes, seconds = divmod(rem, 60)
                elapsedtime_str = "{:0>2}:{:0>2}:{:05.2f}".format(int(hours), int(minutes), seconds) if dt_elapsedtime > 0 else ""

                vector_str = "{}".format(", ".join(format(x, " 5.2e") for x in individual.vector))
                costs_str = "{}".format(", ".join(format(x, " 5.2e") for x in individual.costs)) if state.value == 2 else ""

                print(template.format(state_str, dt_startime, dt_finishtime, elapsedtime_str, vector_str, costs_str))

    time.sleep(1.0)



import setuptools
from setuptools.command.install import install
import subprocess
import os


class CustomInstallCommand(install):
    """Custom install setup to help run shell commands (outside shell) before installation"""

    def run(self):
        print('------------------------OK-------------------------------')
        os.system('pip3 install numpy')
        os.system('pip3 install cython')
        install.run(self)


base_dir = os.path.abspath(os.path.dirname(__file__))

# long description
with open("README.md", "r") as fh:
    long_description = fh.read()

# requirements
with open("requirements.txt", "r") as fh:
    requirements = fh.read().split("\n")

setuptools.setup(
    name="artap",
    version="2020.3.21.1",
    author=u"Artap Team",
    author_email="artap.framework@gmail.com",
    description="Platform for robust design optimization",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="http://www.agros2d.org/artap/",
    python_requires='>3.7',
    license="License :: OSI Approved :: MIT License",
    cmdclass={'install': CustomInstallCommand},
    packages=setuptools.find_packages(),
    include_package_data=True,
    # data_files=[('artap/lib', ['artap/lib/bayesopt.so']),
    install_requires=requirements,
    # scripts=['3rdparty/submodules.sh'],
    classifiers=[
        "Intended Audience :: Science/Research",
        "Operating System :: POSIX :: Linux",
        "Topic :: Scientific/Engineering",
        'Programming Language :: Python :: 3.6',
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
